#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent the Request and its
sub-classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import xml.etree.ElementTree as ET
from typing import List
from abc import ABCMeta, abstractmethod

from sscws.bfieldmodels import BFieldModel
from sscws.filteroptions import LocationFilterOptions, RegionFilterOptions
from sscws.outputoptions import OutputOptions
from sscws.timeinterval import TimeInterval


class Request(metaclass=ABCMeta):
    """
    Class representing a Request from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Notes
    -----
    Although this class is essentially a dictionary, it was defined as a
    class to make certain that it matched the structure and key names
    of a Request from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    It also needs to function as a base class for the concrete
    sub-classes of a Request.

    Properties
    ----------
    description
        Textual description of this request.
    interval
        Time interval of this request
    b_field_model
        Magnetic field model.  If None, default is BFieldModel.
    """
    @abstractmethod
    def __init__(self,
                 description: str,
                 interval: TimeInterval,
                 b_field_model: BFieldModel = None):
        """
        Creates an object representing a Request from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        description
            description of request
        interval
            time interval of request
        b_field_model
            magnetic field model
        """
        self._description = description
        self._interval = interval
        if b_field_model is None:
            self._b_field_model = BFieldModel()
        else:
            self._b_field_model = b_field_model


    @property
    def description(self):
        """
        Gets the description value.

        Returns
        -------
        str
            description value.
        """
        return self._description


    @description.setter
    def description(self, value):
        """
        Sets the description value.

        Parameters
        ----------
        value
            description value.
        """
        self._description = value


    @property
    def interval(self):
        """
        Gets the interval value.

        Returns
        -------
        str
            interval value.
        """
        return self._interval


    @interval.setter
    def interval(self, value):
        """
        Sets the interval value.

        Parameters
        ----------
        value
            interval value.
        """
        self._interval = value


    @property
    def b_field_model(self):
        """
        Gets the b_field_model value.

        Returns
        -------
        str
            b_field_model value.
        """
        return self._b_field_model


    @b_field_model.setter
    def b_field_model(self, value):
        """
        Sets the b_field_model value.

        Parameters
        ----------
        value
            b_field_model value.
        """
        self._b_field_model = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        if isinstance(self, DataRequest):
            request_type = 'Data'
#        elif isinstance(self, GraphRequest):
#            request_type = 'Graph'
#        elif isinstance(self, KmlRequest):
#            request_type = 'Kml'
        elif isinstance(self, LocationRequest):
            request_type = 'Location'
#        elif isinstance(self, QueryRequest):
#            request_type = 'Query'
        else:
            request_type = ''

        builder = ET.TreeBuilder()
        builder.start(request_type + 'Request',
                      {'xmlns': 'http://sscweb.gsfc.nasa.gov/schema'})
        builder.start('Description', {})
        builder.data(self._description)
        builder.end('Description')
        builder.end(request_type + 'Request')
        xml_element = builder.close()
        xml_element.append(self._interval.xml_element())
        xml_element.append(self._b_field_model.xml_element())

        return xml_element


class SatelliteSpecification:
    """
    Class representing a SatelliteSpecification from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    identifier
        satellite identifier
    resolution_factor
        resolution factor
    """
    def __init__(self,
                 identifier: str,
                 resolution_factor: int):
        """
        Creates an object representing a SatelliteSpecification from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        identifier
            satellite identifier
        resolution_factor
            resolution factor
        """
        self._identifier = identifier
        self._resolution_factor = resolution_factor


    @property
    def identifier(self):
        """
        Gets the identifier value.

        Returns
        -------
        str
            identifier value.
        """
        return self._identifier


    @identifier.setter
    def identifier(self, value):
        """
        Sets the identifier value.

        Parameters
        ----------
        value
            identifier value.
        """
        self._identifier = value


    @property
    def resolution_factor(self):
        """
        Gets the resolution_factor value.

        Returns
        -------
        str
            resolution_factor value.
        """
        return self._resolution_factor


    @resolution_factor.setter
    def resolution_factor(self, value):
        """
        Sets the resolution_factor value.

        Parameters
        ----------
        value
            resolution_factor value.
        """
        self._resolution_factor = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('Satellites', {})
        builder.start('Id', {})
        builder.data(self._identifier)
        builder.end('Id')
        builder.start('ResolutionFactor', {})
        builder.data(str(self._resolution_factor))
        builder.end('ResolutionFactor')
        builder.end('Satellites')

        return builder.close()


class LocationRequest(Request):
    """
    Class representing a LocationRequest from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    satellites
        Requested satellites.
    """
    def __init__(self,
                 description: str,
                 interval: TimeInterval,
                 satellites: List[SatelliteSpecification],
                 b_field_model: BFieldModel = None):
        """
        Creates an object representing a LocationRequest from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        description
            Description of request.
        interval
            Time interval of request.
        b_field_model
            Magnetic field model.  If None, default is BFieldModel.
        setellites
            array of SatelliteSpecifications
        """
        super().__init__(description, interval, b_field_model)

        self._satellites = satellites


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        xml_element = super().xml_element()

        for sat in self._satellites:
            xml_element.append(sat.xml_element())

        return xml_element


    @property
    def satellites(self):
        """
        Gets the satellites value.

        Returns
        -------
        str
            satellites value.
        """
        return self._satellites


    @satellites.setter
    def satellites(self, value):
        """
        Sets the satellites value.

        Parameters
        ----------
        value
            satellites value.
        """
        self._satellites = value


class DataRequest(LocationRequest):
    """
    Class representing a DataRequest from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    output_options
        Output options.
    region_filter_options
        Region filter options.
    location_filter_options
        Location filter options.
    """
    def __init__(self,
                 description: str,
                 interval: TimeInterval,
                 satellites: List[SatelliteSpecification],
                 b_field_model: BFieldModel = None,
                 output_options: OutputOptions = None,
                 region_filter_options: RegionFilterOptions = None,
                 location_filter_options: LocationFilterOptions = None
                 ):  # pylint: disable=too-many-arguments
        """
        Creates an object representing a LocationRequest from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        description
            Description of request.
        interval
            Time interval of request.
        satellites
            Array of SatelliteSpecifications.
        b_field_model
            Magnetic field model.  If None, default is BFieldModel.
        output_options
            Output options.
        region_filter_options
            Region filter options.
        location_filter_options
            Location filter options.
        """
        super().__init__(description,
                         interval, satellites, b_field_model)

        self._output_options = output_options
        self._region_filter_options = region_filter_options
        self._location_filter_options = location_filter_options


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        xml_element = super().xml_element()

        if self._output_options is not None:
            xml_element.append(self._output_options.xml_element())
        if self._region_filter_options is not None:
            xml_element.append(self._region_filter_options.xml_element())
        if self._location_filter_options is not None:
            xml_element.append(self._location_filter_options.xml_element())

        return xml_element


    @property
    def output_options(self):
        """
        Gets the output_options value.

        Returns
        -------
        str
            output_options value.
        """
        return self._output_options


    @output_options.setter
    def output_options(self, value):
        """
        Sets the output_options value.

        Parameters
        ----------
        value
            output_options value.
        """
        self._output_options = value


    @property
    def region_filter_options(self):
        """
        Gets the region_filter_options value.

        Returns
        -------
        str
            region_filter_options value.
        """
        return self._region_filter_options


    @region_filter_options.setter
    def region_filter_options(self, value):
        """
        Sets the region_filter_options value.

        Parameters
        ----------
        value
            region_filter_options value.
        """
        self._region_filter_options = value


    @property
    def location_filter_options(self):
        """
        Gets the location_filter_options value.

        Returns
        -------
        str
            location_filter_options value.
        """
        return self._location_filter_options


    @location_filter_options.setter
    def location_filter_options(self, value):
        """
        Sets the location_filter_options value.

        Parameters
        ----------
        value
            location_filter_options value.
        """
        self._location_filter_options = value



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2019 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Example Satellite Situation Center (SSC) web services client.
https://sscweb.gsfc.nasa.gov/WebServices/REST/.

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import sys
import getopt
import json
#import os
#import platform
import xml.etree.ElementTree as ET
#from datetime import datetime, timezone
#from enum import Enum
import logging
import logging.config
from typing import List
import urllib3
#import httplib2
#import dateutil.parser

from sscws.sscws import SscWs
from sscws.bfieldmodels import BFieldModel, ExternalBFieldModel,\
    Tsyganenko89cBFieldModel
from sscws.coordinates import CoordinateComponent, CoordinateSystem
from sscws.filteroptions import LocationFilterOptions,\
    MappedRegionFilterOptions, RegionFilterOptions,\
    SpaceRegionsFilterOptions
from sscws.outputoptions import CoordinateOptions, BFieldTraceOptions,\
    DistanceFromOptions, LocationFilter, OutputOptions, RegionOptions,\
    ValueOptions
from sscws.regions import FootpointRegion, Hemisphere, HemisphereRegions,\
    SpaceRegion
from sscws.request import DataRequest, SatelliteSpecification
from sscws.timeinterval import TimeInterval


logging.basicConfig()
LOGGING_CONFIG_FILE = 'logging_config.json'
try:
    with open(LOGGING_CONFIG_FILE, 'r') as fd:
        logging.config.dictConfig(json.load(fd))
except BaseException as exc:    # pylint: disable=broad-except
    if not isinstance(exc, FileNotFoundError):
        print('Logging configuration failed')
        print('Exception: ', exc)
        print('Ignoring failure')
        print()


#ENDPOINT = "https://sscweb.sci.gsfc.nasa.gov/WS/sscr/2/"
ENDPOINT = "http://sscweb-dev.sci.gsfc.nasa.gov/WS/sscr/2/"
#ENDPOINT = "http://localhost:8383/WS/sscr/2/"
#CA_CERTS = '/etc/pki/ca-trust/extracted/openssl/ca-bundle.trust.crt'


def print_usage(
        name: str
    ) -> None:
    """
    Prints program usage information to stdout.

    Parameters
    ----------
    name
        name of this program

    Returns
    -------
    None
    """
    print('USAGE: {name} [-e url][-d][-c cacerts][-h]'.format(name=name))
    print('WHERE: url = SSC web service endpoint URL')
    print('       -d disables TLS server certificate validation')
    print('       cacerts = CA certificate filename')


def example(
        argv: List[str]
    ) -> None: # pylint: disable=too-many-locals,too-many-branches,too-many-statements
    """
    Example Coordinate Data Analysis System (CDAS) web service client.
    Includes example calls to most of the web services.

    Parameters
    ----------
    argv
        Command-line arguments.<br>
        -e url or --endpoint=url where url is the cdas web service endpoint
            URL to use.<br>
        -c url or --cacerts=filename where filename is the name of the file
            containing the CA certificates to use.<br>
        -d or --disable-cert-check to disable verification of the server's
            certificate
        -h or --help prints help information.
    """

    try:
        opts = getopt.getopt(argv[1:], 'he:c:d',
                             ['help', 'endpoint=', 'cacerts=',
                              'disable-cert-check'])[0]
    except getopt.GetoptError:
        print('ERROR: invalid option')
        print_usage(argv[0])
        sys.exit(2)

    endpoint = ENDPOINT
    ca_certs = None
    disable_ssl_certificate_validation = False

    for opt, arg in opts:
        if opt in ('-e', '--endpoint'):
            endpoint = arg
        elif opt in ('-c', '--cacerts'):
            ca_certs = arg
        elif opt in ('-d', '--disable-cert-check'):
            disable_ssl_certificate_validation = True
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        elif opt in ('-h', '--help'):
            print_usage(argv[0])
            sys.exit()

    ssc = SscWs(endpoint=endpoint, ca_certs=ca_certs,
                disable_ssl_certificate_validation=
                disable_ssl_certificate_validation)

#    print('SSC Observatories:')
#    for observatory in ssc.get_observatories():
#
#        # this needs to be prettier
#        print(observatory['Id'], observatory['Name'],
#              observatory['StartTime'])
#
#    print('SSC Ground Stations:')
#    for ground_station in ssc.get_ground_stations():
#
#        location = ground_station['Location']
#
#        # this needs to be prettier
#        print(ground_station['Id'], ground_station['Name'],
#              location['Latitude'], location['Longitude'])

    example_request = create_example_request()
    #print('request:')
    #print(ET.tostring(example_request.xml_element()).decode('utf-8'))
    status, result = ssc.get_locations(example_request)

    if status == 200:

        print_locations_result(result)
    else:
        print('ssc.get_locations failed with status = ', status)
        print('request:')
        print(ET.tostring(example_request.xml_element()).decode('utf-8'))


def create_example_request(
    ) -> ET:
    """
    Create an example DataRequest.

    Returns
    -------
    ET
        ElementTree representation of an example DataRequest.
    """
    sats = [SatelliteSpecification('themisa', 2),
            SatelliteSpecification('themisb', 2)]
    b_field_model = BFieldModel(external=Tsyganenko89cBFieldModel())
    coord_options = [
        CoordinateOptions(CoordinateSystem.GSE, CoordinateComponent.X),
        CoordinateOptions(CoordinateSystem.GSE, CoordinateComponent.Y),
        CoordinateOptions(CoordinateSystem.GSE, CoordinateComponent.Z),
        CoordinateOptions(CoordinateSystem.GSE, CoordinateComponent.LAT),
        CoordinateOptions(CoordinateSystem.GSE, CoordinateComponent.LON),
        CoordinateOptions(CoordinateSystem.GSE, CoordinateComponent.LOCAL_TIME)
        ]
    b_field_trace_options = [
        BFieldTraceOptions(CoordinateSystem.GEO, Hemisphere.NORTH,
                           True, True, True),
        BFieldTraceOptions(CoordinateSystem.GEO, Hemisphere.SOUTH,
                           True, True, True)
        ]

    output_options = OutputOptions(
        coord_options,
        None, None,
        RegionOptions(True, True, True, True),
        ValueOptions(True, True, True, True),
        DistanceFromOptions(True, True, True, True),
        b_field_trace_options
        )
    loc_filter = LocationFilter(0, 100000, True, True)
    loc_filter_options = LocationFilterOptions(True, loc_filter,
                                               loc_filter, loc_filter,
                                               loc_filter, loc_filter,
                                               loc_filter, loc_filter)

    hemisphere_region = HemisphereRegions(True, True)
    trace_regions = MappedRegionFilterOptions(hemisphere_region,
                                              hemisphere_region,
                                              hemisphere_region,
                                              hemisphere_region,
                                              hemisphere_region,
                                              True)
    # improved the following ???
    srfo = SpaceRegionsFilterOptions()

    rfo = RegionFilterOptions(srfo, trace_regions, trace_regions)

    loc_request = DataRequest('Example locator request.',
                              TimeInterval('2020-10-02T00:00:00Z',
                                           '2020-10-02T01:00:00Z'),
                              sats, b_field_model,
                              output_options, rfo,
                              None)
#                              loc_filter_options)
    return loc_request
#    data_request = {
#        'Description': 'Example locator request.',
#        'TimeInterval': {
#            'Start': datetime(2008, 1, 2, 11, 0, 0, tzinfo=timezone.utc),
#            'End': datetime(2008, 1, 2, 11, 59, 59, tzinfo=timezone.utc)
#        },
#        'BFieldModel': {
#            'InternalBFieldModel': InternalBFieldModel.IGRF,
#            'ExternalBFieldModel': {
#                'Name': ExternalBFieldModel.TSYGANENKO89C,
#                'KeyParameterValues': Tsyganenko89cKp.KP_3_3_3
#            }
#        },
#        'Satellites': [],
#        'OutputOptions': {
#            'AllLocationFilters': True,
#            'CoordinateOptions': [],
#            'RegionOptions': {
#                'Spacecraft': True,
#                'RadialTracedFootpoint': True,
#                'NorthBTracedFootpoint': True,
#                'SouthBTracedFootpoint': True
#            },
#            'ValueOptions': {
#                'RadialDistance': True,
#                'BFieldStrength': True,
#                'DipoleLValue': True,
#                'DipoleInvLat': True
#            },
#            'DistanceFromOptions': {
#                'NeutralSheet': True,
#                'BowShock': True,
#                'MPause': True,
#                'BGseXYZ': True
#            },
#            'BFieldTraceOptions': [{
#                'CoordinateSystem': CoordinateSystem.GEO.value,
#                'Hemisphere': Hemisphere.NORTH.value,
#                'FootpointLatitude': True,
#                'FootpointLongitude': True,
#                'FieldLineLength': True
#            }, {
#                'CoordinateSystem': CoordinateSystem.GEO.value,
#                'Hemisphere': Hemisphere.SOUTH.value,
#                'FootpointLatitude': True,
#                'FootpointLongitude': True,
#                'FieldLineLength': True,
#            }]
#        },
#        'RegionFilterOptions': {
#            'SpaceRegions': {
#                'InterplanetaryMedium': True,
#                'DaysideMagnetosheath': True,
#                'NightsideMagnetosheath': True,
#                'DaysideMagnetosphere': True,
#                'NightsideMagnetosphere': True,
#                'PlasmaSheet': True,
#                'TailLobe': True,
#                'HighLatitudeBoundaryLayer': True,
#                'LowLatitudeBoundaryLayer': True,
#                'DaysidePlasmasphere': True,
#                'NightsidePlasmasphere': True
#            },
#            'RadialTraceRegions': {
#                'Cusp': {
#                    'North': True,
#                    'South': True
#                },
#                'Cleft': {
#                    'North': True,
#                    'South': True
#                },
#                'AuroralOval': {
#                    'North': True,
#                    'South': True
#                },
#                'PolarCap': {
#                    'North': True,
#                    'South': True
#                },
#                'MidLatitude': {
#                    'North': True,
#                    'South': True
#                },
#                'LowLatitude': True
#            },
#            'MagneticTraceRegions': {
#                'Cusp': {
#                    'North': True,
#                    'South': True
#                },
#                'Cleft': {
#                    'North': True,
#                    'South': True
#                },
#                'AuroralOval': {
#                    'North': True,
#                    'South': True
#                },
#                'PolarCap': {
#                    'North': True,
#                    'South': True
#                },
#                'MidLatitude': {
#                    'North': True,
#                    'South': True
#                },
#                'LowLatitude': True
#            }
#        }
#    }
#    for sat in ['themisa', 'spase://SMWG/Observatory/THEMIS/B']:
#        data_request['Satellites'].append({
#            'Id': sat,
#            'ResolutionFactor': 2
#        })
#    for component in [CoordinateComponent.X.value,
#                      CoordinateComponent.Y.value,
#                      CoordinateComponent.Z.value,
#                      CoordinateComponent.LAT.value,
#                      CoordinateComponent.LON.value,
#                      CoordinateComponent.LOCAL_TIME.value]:
#        data_request['OutputOptions']['CoordinateOptions'].append({
#            'CoordinateSystem': CoordinateSystem.GSE.value,
#            'Component': component
#        })
#
#    data_request['OutputOptions']['MinMaxPoints'] = 2
#
#    return data_request



def print_locations_result(result):
    """
    Prints a Result document.

    :param result: dict representation of Result as described in SSC.xsd.
    """

    print('StatusCode:', result['StatusCode'],
          'StatusSubCode:', result['StatusSubCode'])
    #print(result)

    for data in result['Data']:
        if 'Coordinates' not in data:
            continue
        coords = data['Coordinates']
        print(data['Id'], coords['CoordinateSystem'].value)
        print('Time                     ', 'X                     ',
              'Y                     ', 'Z                     ')
        for index in range(min(len(data['Time']), len(coords['X']))):
            print(data['Time'][index], coords['X'][index],
                  coords['Y'][index], coords['Z'][index])

        for b_trace in data['BTraceData']:

            print(b_trace['CoordinateSystem'].value,
                  b_trace['Hemisphere'].value)
            print('Time                          ', 'Latitude        ',
                  'Longitude   ', 'Arc Length')
            for index in range(min(len(data['Time']),
                                   len(b_trace['Latitude']))):
                print(data['Time'][index],
                      '{:15.5f} {:15.5f} {:15.5f}'.format(\
                          b_trace['Latitude'][index],
                          b_trace['Longitude'][index],
                          b_trace['ArcLength'][index]))

        if 'RadialLength' in data:
            print('Time                     ', 'Radial Length         ')
            for index in range(min(len(data['Time']),
                                   len(data['RadialLength']))):
                print(data['Time'][index], data['RadialLength'][index])

        # repeat for MagneticStrength, NeutralSheetDistance,
        # BowShockDistance, MagnetoPauseDistance, DipoleLValue,
        # DipoleInvariantLatitude, SpacecraftRegion,
        # RadialTracedFootpointRegions

        if 'BGseX' in data:

            print('Time                     ', 'B Strength GSE        ')
            print('                             X                     ',
                  'Y                     ', 'Z')
            for index in range(min(len(data['Time']), len(data['BGseX']))):
                print(data['Time'][index], data['BGseX'][index],
                      data['BGseY'][index],
                      data['BGseZ'][index])

        if 'NorthBTracedFootpointRegion' in data and \
           'SouthBTracedFootpointRegion' in data:

            print('                 B-Traced Footpoint Region')
            print('Time                     ', 'North            ',
                  'South           ')
            for index in range(min(len(data['Time']),
                                   len(data['NorthBTracedFootpointRegion']))):
                print(data['Time'][index],
                      data['NorthBTracedFootpointRegion'][index].value,
                      data['SouthBTracedFootpointRegion'][index].value)

def main():
    """
    Example Satellite Situation Center SSC web service client.
    """
#    for observatory in get_observatories():
#
#        print(observatory['Id'], observatory['Name'],
#              observatory['StartTime'])
#
#   print_locations_result(
#       get_locations(
#           create_example_request()))


if __name__ == '__main__':
    #main()
    example(sys.argv)



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent region classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import xml.etree.ElementTree as ET
from enum import Enum


class FootpointRegion(Enum):
    """
    Enumerations representing the FootpointRegion defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    NOT_APPLICABLE = 'NotApplicable'
    NORTH_CUSP = 'NorthCusp'
    SOUTH_CUSP = 'SouthCusp'
    NORTH_CLEFT = 'NorthCleft'
    SOUTH_CLEFT = 'SouthCleft'
    NORTH_AURORAL_OVAL = 'NorthAuroralOval'
    SOUTH_AURORAL_OVAL = 'SouthAuroralOval'
    NORTH_POLAR_CAP = 'NorthPolarCap'
    SOUTH_POLAR_CAP = 'SouthPolarCap'
    NORTH_MID_LATITUDE = 'NorthMidLatitude'
    SOUTH_MID_LATITUDE = 'SouthMidLatitude'
    LOW_LATITUDE = 'LowLatitude'


class Hemisphere(Enum):
    """
    Enumerations representing the Hemisphere defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    SOUTH = 'South'
    NORTH = 'North'


class HemisphereRegions:
    """
    Class representing a HemisphereRegions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    north
        Northern hemisphere region.
    south
        Southern hemisphere region.
    """
    def __init__(self,
                 north: bool,
                 south: bool):
        """
        Creates an object representing a HemisphereRegions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        north
            Northern hemisphere region.
        south
            Southern hemisphere region.
        """
        self._north = north
        self._south = south


    @property
    def north(self):
        """
        Gets the north value.

        Returns
        -------
        str
            north value.
        """
        return self._north


    @north.setter
    def north(self, value):
        """
        Sets the north value.

        Parameters
        ----------
        value
            north value.
        """
        self._north = value


    @property
    def south(self):
        """
        Gets the south value.

        Returns
        -------
        str
            south value.
        """
        return self._south


    @south.setter
    def south(self, value):
        """
        Sets the south value.

        Parameters
        ----------
        value
            south value.
        """
        self._south = value


    def xml_element(self,
                    name: str) -> ET:
        """
        Produces the XML Element representation of this object.

        Parameters
        ----------
        name
            Name of Region.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start(name, {})
        builder.start('North', {})
        builder.data(str(self._north).lower())
        builder.end('North')
        builder.start('South', {})
        builder.data(str(self._south).lower())
        builder.end('South')
        builder.end(name)

        return builder.close()


class SpaceRegion(Enum):
    """
    Enumerations representing the SpaceRegion defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    INTERPLANETARY_MEDIUM = 'InterplanetaryMedium'
    DAYSIDE_MAGNETOSHEATH = 'DaysideMagnetosheath'
    NIGHTSIDE_MAGNETOSHEATH = 'NightsideMagnetosheath'
    DAYSIDE_MAGNETOSPHERE = 'DaysideMagnetosphere'
    NIGHTSIDE_MAGNETOSPHERE = 'NightsideMagnetosphere'
    PLASMA_SHEET = 'PlasmaSheet'
    TAIL_LOBE = 'TailLobe'
    LOW_LATITUDE_BOUNDARY_LAYER = 'LowLatitudeBoundaryLayer'
    HIGH_LATITUDE_BOUNDARY_LAYER = 'HighLatitudeBoundarLayer'
    DAYSIDE_PLASMASPHERE = 'DaysidePlasmasphere'
    NIGHTSIDE_PLASMASPHERE = 'NightsidePlasmasphere'


class SpaceRegionType(Enum):
    """
    Enumerations representing the SpaceRegionType defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    INTERPLANETARY_MEDIUM = 'InterplanetaryMedium'
    DAYSIDE_MAGNETOSHEATH = 'DaysideMagnetosheath'
    NIGHTSIDE_MAGNETOSHEATH = 'NightsideMagnetosheath'
    DAYSIDE_MAGNETOSPHERE = 'DaysideMagnetosphere'
    NIGHTSIDE_MAGNETOSPHERE = 'NightsideMagnetosphere'
    PLASMA_SHEET = 'PlasmaSheet'
    TAIL_LOBE = 'TailLobe'
    LOW_LATITUDE_BOUNDARY_LAYER = 'LowLatitudeBoundaryLayer'
    HIGH_LATITUDE_BOUNDARY_LAYER = 'HighLatitudeBoundarLayer'
    DAYSIDE_PLASMASPHERE = 'DaysidePlasmasphere'
    NIGHTSIDE_PLASMASPHERE = 'NightsidePlasmasphere'



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2018-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#


"""
Module defining a class to represent the TimeInterval class from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2018-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""


import xml.etree.ElementTree as ET
from datetime import datetime, timezone
from typing import Tuple, Union
import dateutil.parser




class TimeInterval:
    """
    A class representing a TimeInterval class from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    start
        Start time of interval.
    end
        End time of interval.
    """
    def __init__(self, start: Union[datetime, str],
                 end: Union[datetime, str]):
        """
        Constructs a TimeInterval object.

        Parameters
        ----------
        start
            Start time of interval.
        end
            End time of interval.
        Raises
        ------
        ValueError
            If the given start/end datetime values are invalid.
        """

        self._start = TimeInterval.get_datetime(start)
        self._end = TimeInterval.get_datetime(end)


    @property
    def start(self):
        """
        Gets the start value.

        Returns
        -------
        datetime
            start value.
        """
        return self._start


    @start.setter
    def start(self, value):
        """
        Sets the start value.

        Parameters
        ----------
        value
            start datetime value.
        """
        self._start = TimeInterval.get_datetime(value)


    @property
    def end(self):
        """
        Gets the end value.

        Returns
        -------
        datetime
            end value.
        """
        return self._end


    @end.setter
    def end(self, value):
        """
        Sets the _end value.

        Parameters
        ----------
        value
            end datetime value.
        """
        self._end = TimeInterval.get_datetime(value)


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('TimeInterval', {})
        builder.start('Start', {})
        builder.data(self._start.isoformat())
        builder.end('Start')
        builder.start('End', {})
        builder.data(self._end.isoformat())
        builder.end('End')
        builder.end('TimeInterval')

        return builder.close()


    def __str__(self):
        return self._start.isoformat() + ' ' + self._end.isoformat()


    def __eq__(self, other):
        return self._start == other.start and self._end == other.end


    @staticmethod
    def get_datetime(value: Union[datetime, str]) -> datetime:
        """
        Produces a datetime representation of the given value.

        Parameters
        ----------
        value
            value to convert to a datetime.
        Returns
        -------
        datetime
            datetime representation of the given value.
        Raises
        ------
        ValueError
            If the given value is not a valid datetime value.
        """
        if isinstance(value, datetime):
            datetime_value = value
        elif isinstance(value, str):
            datetime_value = dateutil.parser.parse(value)
        else:
            raise ValueError('unrecognized datetime value')

        datetime_value.astimezone(timezone.utc)

        return datetime_value


    @staticmethod
    def get_datetimes(
            start: Union[datetime, str],
            end: Union[datetime, str]
        ) -> Tuple[datetime, datetime]:
        """
        Produces a datetime representation of the given values.

        Parameters
        ----------
        start
            start value to convert to a datetime.
        end
            end value to convert to a datetime.
        Returns
        -------
        Tuple
            [0] datetime representation of the given start value.
            [1] datetime representation of the given end value.
        Raises
        ------
        ValueError
            If either of the given values is not a valid datetime value.
        """
        try:
            start_datetime = TimeInterval.get_datetime(start)
        except ValueError:
            raise ValueError('unrecognized start datetime value')

        try:
            end_datetime = TimeInterval.get_datetime(end)
        except ValueError:
            raise ValueError('unrecognized end datetime value')

        return start_datetime, end_datetime


    @staticmethod
    def basic_iso_format(value: datetime) -> str:
        """
        Produces the basic (minimal) ISO 8601 format of the given
        datetime.

        Parameters
        ----------
        value
            datetime value to convert to string.
        Returns
        -------
        str
            Basic ISO 8601 format time string.
        """
        return value.isoformat().replace('+00:00', 'Z').translate(
            {ord(i):None for i in ':-'})



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent the mapping classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

from enum import Enum


class MapProjection(Enum):
    """
    Enumerations representing the MapProjection defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    AZIMUTHAL = 'Azimuthal'
    CYLINDRICAL = 'Cylindrical'
    MERCATOR = 'Mercator'
    MOLLEWEIDE = 'Molleweide'
    ORTHOGRAPHIC = 'Othographic'
    STEREOGRAPHIC = 'Stereographic'


class MapProjectionTrace(Enum):
    """
    Enumerations representing the MapProjectionTrace defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    B_FIELD_NORTH = 'BFieldNorth'
    B_FIELD_SOUTH = 'BFieldSouth'
    RADIAL = 'Radial'


class MapRegion(Enum):
    """
    Enumerations representing the MapRegion defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    NORTH_CUSP = 'NorthCusp'
    SOUTH_CUSP = 'SouthCusp'
    NORTH_CLEFT = 'NorthCleft'
    SOUTH_CLEFT = 'SouthCleft'
    NORTH_AURORAL_OVAL = 'NorthAuroralOval'
    SOUTH_AURORAL_OVAL = 'SouthOuroralOval'
    NORTH_POLAR_CAP = 'NorthPolarCap'
    SOUTH_POLAR_CAP = 'SouthPolarCap'
    NORTH_MID_LATITUDE = 'NorthMidLatitude'
    SOUTH_MID_LATITUDE = 'SouthMidLatitude'
    LOW_LATITUDE = 'LowLatitude'
    NONE = 'None'


class PolarMapOrientation(Enum):
    """
    Enumerations representing the PolarMapOrientation defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    EQUATORIAL = 'Equatorial'
    NORTH_POLE = 'NorthPole'
    SOUTH_POLE = 'SouthPole'



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent the Request and its
sub-classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import xml.etree.ElementTree as ET
from typing import List

from sscws.coordinates import CoordinateSystem, CoordinateComponent
from sscws.regions import Hemisphere


class CoordinateOptions:
    """
    Class representing a CoordinateOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    coordinate_system
        Coordinate system.
    component
        Coordinate system component.
    """
    def __init__(self,
                 coordinate_system: CoordinateSystem,
                 component: CoordinateComponent):
        """
        Creates an object representing a CoordinateOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        coordinate_system
            Coordinate system.
        component
            Coordinate system component.
        """
        self._coordinate_system = coordinate_system
        self._component = component


    @property
    def coordinate_system(self):
        """
        Gets the coordinate_system value.

        Returns
        -------
        str
            coordinate_system value.
        """
        return self._coordinate_system


    @coordinate_system.setter
    def coordinate_system(self, value):
        """
        Sets the coordinate_system value.

        Parameters
        ----------
        value
            coordinate_system value.
        """
        self._coordinate_system = value


    @property
    def component(self):
        """
        Gets the component value.

        Returns
        -------
        str
            component value.
        """
        return self._component


    @component.setter
    def component(self, value):
        """
        Sets the component value.

        Parameters
        ----------
        value
            component value.
        """
        self._component = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('CoordinateOptions', {})
        builder.start('CoordinateSystem', {})
        builder.data(self._coordinate_system.value)
        builder.end('CoordinateSystem')
        builder.start('Component', {})
        builder.data(self._component.value)
        builder.end('Component')
        builder.end('CoordinateOptions')

        return builder.close()


class LocationFilter:
    """
    Class representing a LocationFilter from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    minimum
        Specifies whether the minimum value is to be marked by a "v"
        in the listing.
    maximum
        Specifies whether the maximum value is to be marked by a "v"
        in the listing.
    lower_limit
        Specifies the lower limit of values that are to be included in
        the listing.  The value is kilometer or degrees as applicable.
    upper_limit
        Specifies the upper limit of values that are to be included in
        the listing.  The value is kilometer or degrees as applicable.
    """
    def __init__(self,
                 lower_limit: float,
                 upper_limit: float,
                 minimum: bool = None,
                 maximum: bool = None):
        """
        Creates an object representing a LocationFilter from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        minimum
            Specifies whether the minimum value is to be marked by a "v"
            in the listing.  If None, default is False.
        maximum
            Specifies whether the maximum value is to be marked by a "v"
            in the listing.  If None, default is False.
        lower_limit
            Specifies the lower limit of values that are to be included in
            the listing.  The value is kilometer or degrees as applicable.
        upper_limit
            Specifies the upper limit of values that are to be included in
            the listing.  The value is kilometer or degrees as applicable.
        """

        if minimum is None:
            self._minimum = False
        else:
            self._minimum = minimum

        if maximum is None:
            self._maximum = False
        else:
            self._maximum = maximum

        self._lower_limit = lower_limit
        self._upper_limit = upper_limit


    @property
    def minimum(self):
        """
        Gets the minimum value.

        Returns
        -------
        str
            minimum value.
        """
        return self._minimum


    @minimum.setter
    def minimum(self, value):
        """
        Sets the minimum value.

        Parameters
        ----------
        value
            minimum value.
        """
        self._minimum = value


    @property
    def maximum(self):
        """
        Gets the maximum value.

        Returns
        -------
        str
            maximum value.
        """
        return self._maximum


    @maximum.setter
    def maximum(self, value):
        """
        Sets the maximum value.

        Parameters
        ----------
        value
            maximum value.
        """
        self._maximum = value


    @property
    def lower_limit(self):
        """
        Gets the lower_limit value.

        Returns
        -------
        str
            lower_limit value.
        """
        return self._lower_limit


    @lower_limit.setter
    def lower_limit(self, value):
        """
        Sets the lower_limit value.

        Parameters
        ----------
        value
            lower_limit value.
        """
        self._lower_limit = value


    @property
    def upper_limit(self):
        """
        Gets the upper_limit value.

        Returns
        -------
        str
            upper_limit value.
        """
        return self._upper_limit


    @upper_limit.setter
    def upper_limit(self, value):
        """
        Sets the upper_limit value.

        Parameters
        ----------
        value
            upper_limit value.
        """
        self._upper_limit = value


    def xml_element(self,
                    name: str) -> ET:
        """
        Produces the XML Element representation of this object.

        Parameters
        ----------
        name
            Name of this LocationFilter.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start(name, {})
        builder.start('Minimum', {})
        builder.data(str(self._minimum).lower())
        builder.end('Minimum')
        builder.start('Maximum', {})
        builder.data(str(self._maximum).lower())
        builder.end('Maximum')
        builder.start('LowerLimit', {})
        builder.data(str(self._lower_limit))
        builder.end('LowerLimit')
        builder.start('UpperLimit', {})
        builder.data(str(self._upper_limit))
        builder.end('UpperLimit')
        builder.end(name)

        return builder.close()


class FilteredCoordinateOptions(CoordinateOptions):
    """
    Class representing a FilteredCoordinateOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    location_filter
        Filter criteria.
    """
    def __init__(self,
                 coordinate_system: CoordinateSystem,
                 component: CoordinateComponent,
                 location_filter: LocationFilter = None):
        """
        Creates an object representing a FilteredCoordinateOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        coordinate_system
            Coordinate system.
        component
            Coordinate system component.
        location_filter
            Location filter listing options.
        """

        super().__init__(coordinate_system, component)
        self._location_filter = location_filter


    @property
    def location_filter(self):
        """
        Gets the location_filter value.

        Returns
        -------
        str
            location_filter value.
        """
        return self._location_filter


    @location_filter.setter
    def location_filter(self, value):
        """
        Sets the location_filter value.

        Parameters
        ----------
        value
            location_filter value.
        """
        self._location_filter = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        xml_element = super().xml_element()

        if self._location_filter is not None:
            xml_element.append(self._location_filter.xml_element())

        return xml_element


class RegionOptions:
    """
    Class representing a RegionOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    spacecraft
        Specifies whether the spacecraft regions are to be included
        in the listing.
    radial_traced_footpoint
        Specifies whether the radial traced footpoint values are to be
        included in the listing.
    north_b_traced_footpoint
        Specifies the north B-field traced footpoint values that are to be
        included in the listing.
    south_b_traced_footpoint
        Specifies the south B-field traced footpoint values that are to be
        included in the listing.
    """
    def __init__(self,
                 spacecraft: bool = None,
                 radial_traced_footpoint: bool = None,
                 north_b_traced_footpoint: bool = None,
                 south_b_traced_footpoint: bool = None):
        """
        Creates an object representing a RegionOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        spacecraft
            Specifies whether the spacecraft regions are to be included
            in the listing.  If None, default is False.
        radial_traced_footpoint
            Specifies whether the radial traced footpoint values are to be
            included in the listing.  If None, default is False.
        north_b_traced_footpoint
            Specifies the north B-field traced footpoint values that are
            to be included in the listing.  If None, default is False.
        south_b_traced_footpoint
            Specifies the south B-field traced footpoint values that are
            to be included in the listing.  If None, default is False.
        """

        if spacecraft is None:
            self._spacecraft = False
        else:
            self._spacecraft = spacecraft

        if radial_traced_footpoint is None:
            self._radial_traced_footpoint = False
        else:
            self._radial_traced_footpoint = radial_traced_footpoint

        if north_b_traced_footpoint is None:
            self._north_b_traced_footpoint = False
        else:
            self._north_b_traced_footpoint = north_b_traced_footpoint

        if south_b_traced_footpoint is None:
            self._south_b_traced_footpoint = False
        else:
            self._south_b_traced_footpoint = south_b_traced_footpoint


    @property
    def spacecraft(self):
        """
        Gets the spacecraft value.

        Returns
        -------
        str
            spacecraft value.
        """
        return self._spacecraft


    @spacecraft.setter
    def spacecraft(self, value):
        """
        Sets the spacecraft value.

        Parameters
        ----------
        value
            spacecraft value.
        """
        self._spacecraft = value


    @property
    def radial_traced_footpoint(self):
        """
        Gets the radial_traced_footpoint value.

        Returns
        -------
        str
            radial_traced_footpoint value.
        """
        return self._radial_traced_footpoint


    @radial_traced_footpoint.setter
    def radial_traced_footpoint(self, value):
        """
        Sets the radial_traced_footpoint value.

        Parameters
        ----------
        value
            radial_traced_footpoint value.
        """
        self._radial_traced_footpoint = value


    @property
    def north_b_traced_footpoint(self):
        """
        Gets the north_b_traced_footpoint value.

        Returns
        -------
        str
            north_b_traced_footpoint value.
        """
        return self._north_b_traced_footpoint


    @north_b_traced_footpoint.setter
    def north_b_traced_footpoint(self, value):
        """
        Sets the north_b_traced_footpoint value.

        Parameters
        ----------
        value
            north_b_traced_footpoint value.
        """
        self._north_b_traced_footpoint = value


    @property
    def south_b_traced_footpoint(self):
        """
        Gets the south_b_traced_footpoint value.

        Returns
        -------
        str
            south_b_traced_footpoint value.
        """
        return self._south_b_traced_footpoint


    @south_b_traced_footpoint.setter
    def south_b_traced_footpoint(self, value):
        """
        Sets the south_b_traced_footpoint value.

        Parameters
        ----------
        value
            south_b_traced_footpoint value.
        """
        self._south_b_traced_footpoint = value


    def xml_element(
            self
        ) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('RegionOptions', {})
        builder.start('Spacecraft', {})
        builder.data(str(self._spacecraft).lower())
        builder.end('Spacecraft')
        builder.start('RadialTracedFootpoint', {})
        builder.data(str(self._radial_traced_footpoint).lower())
        builder.end('RadialTracedFootpoint')
        builder.start('NorthBTracedFootpoint', {})
        builder.data(str(self._north_b_traced_footpoint).lower())
        builder.end('NorthBTracedFootpoint')
        builder.start('SouthBTracedFootpoint', {})
        builder.data(str(self._south_b_traced_footpoint).lower())
        builder.end('SouthBTracedFootpoint')
        builder.end('RegionOptions')

        return builder.close()


class ValueOptions:
    """
    Class representing a ValueOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    radial_distance
        Specifies whether the radial distance is to be included
        in the listing.
    b_field_strength
        Specifies whether the B field strength values are to be
        included in the listing.
    dipole_l_value
        Specifies whether the dipole L values that are to be
        included in the listing.
    dipole_inv_lat
        Specifies whether the dipole invariant latitude values that
        are to be included in the listing.
    """
    def __init__(self,
                 radial_distance: bool = None,
                 b_field_strength: bool = None,
                 dipole_l_value: bool = None,
                 dipole_inv_lat: bool = None):
        """
        Creates an object representing a ValueOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        radial_distance
            Specifies whether the radial distance values are to be included
            in the listing.  If None, default is False.
        b_field_strength
            Specifies whether the B field strength values are to be
            included in the listing.  If None, default is False.
        dipole_l_value
            Specifies whether the dipole L values that are
            to be included in the listing.  If None, default is False.
        dipole_inv_lat
            Specifies whether the dipole invariant latitude values that
            are to be included in the listing.  If None, default is False.
        """

        if radial_distance is None:
            self._radial_distance = False
        else:
            self._radial_distance = radial_distance

        if b_field_strength is None:
            self._b_field_strength = False
        else:
            self._b_field_strength = b_field_strength

        if dipole_l_value is None:
            self._dipole_l_value = False
        else:
            self._dipole_l_value = dipole_l_value

        if dipole_inv_lat is None:
            self._dipole_inv_lat = False
        else:
            self._dipole_inv_lat = dipole_inv_lat


    @property
    def radial_distance(self):
        """
        Gets the radial_distance value.

        Returns
        -------
        str
            radial_distance value.
        """
        return self._radial_distance


    @radial_distance.setter
    def radial_distance(self, value):
        """
        Sets the radial_distance value.

        Parameters
        ----------
        value
            radial_distance value.
        """
        self._radial_distance = value


    @property
    def b_field_strength(self):
        """
        Gets the b_field_strength value.

        Returns
        -------
        str
            b_field_strength value.
        """
        return self._b_field_strength


    @b_field_strength.setter
    def b_field_strength(self, value):
        """
        Sets the b_field_strength value.

        Parameters
        ----------
        value
            b_field_strength value.
        """
        self._b_field_strength = value


    @property
    def dipole_l_value(self):
        """
        Gets the dipole_l_value value.

        Returns
        -------
        str
            dipole_l_value value.
        """
        return self._dipole_l_value


    @dipole_l_value.setter
    def dipole_l_value(self, value):
        """
        Sets the dipole_l_value value.

        Parameters
        ----------
        value
            dipole_l_value value.
        """
        self._dipole_l_value = value


    @property
    def dipole_inv_lat(self):
        """
        Gets the dipole_inv_lat value.

        Returns
        -------
        str
            dipole_inv_lat value.
        """
        return self._dipole_inv_lat


    @dipole_inv_lat.setter
    def dipole_inv_lat(self, value):
        """
        Sets the dipole_inv_lat value.

        Parameters
        ----------
        value
            dipole_inv_lat value.
        """
        self._dipole_inv_lat = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('ValueOptions', {})
        builder.start('RadialDistance', {})
        builder.data(str(self._radial_distance).lower())
        builder.end('RadialDistance')
        builder.start('BFieldStrength', {})
        builder.data(str(self._b_field_strength).lower())
        builder.end('BFieldStrength')
        builder.start('DipoleLValue', {})
        builder.data(str(self._dipole_l_value).lower())
        builder.end('DipoleLValue')
        builder.start('DipoleInvLat', {})
        builder.data(str(self._dipole_inv_lat).lower())
        builder.end('DipoleInvLat')
        builder.end('ValueOptions')

        return builder.close()


class DistanceFromOptions:
    """
    Class representing a DistanceFromOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    neutral_sheet
        Specifies whether the distance to the neutral sheet values are
        to be included in the listing.
    bow_shock
        Specifies whether the distance to the bow shock values are to be
        included in the listing.
    m_pause
        Specifies whether the distance to the magneto pause values that
        are to be included in the listing.
    b_gse_xyz
        Specifies whether the magnetic field strength in the GSE XYZ
        directions that are to be included in the listing.
    """
    def __init__(self,
                 neutral_sheet: bool = None,
                 bow_shock: bool = None,
                 m_pause: bool = None,
                 b_gse_xyz: bool = None):
        """
        Creates an object representing a DistanceFromOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        neutral_sheet
            Specifies whether the distance to the neutral sheet values are
            to be included in the listing.  If None, default is False.
        bow_shock
            Specifies whether the distance to the bow shock values are to
            be included in the listing.  If None, default is False.
        m_pause
            Specifies whether the distance to the magneto pause values that
            are to be included in the listing.  If None, default is False.
        b_gse_xyz
            Specifies whether the magnetic field strength in the GSE XYZ
            directions that are to be included in the listing.  If None,
            default is False.
        """

        if neutral_sheet is None:
            self._neutral_sheet = False
        else:
            self._neutral_sheet = neutral_sheet

        if bow_shock is None:
            self._bow_shock = False
        else:
            self._bow_shock = bow_shock

        if m_pause is None:
            self._m_pause = False
        else:
            self._m_pause = m_pause

        if b_gse_xyz is None:
            self._b_gse_xyz = False
        else:
            self._b_gse_xyz = b_gse_xyz


    @property
    def neutral_sheet(self):
        """
        Gets the neutral_sheet value.

        Returns
        -------
        str
            neutral_sheet value.
        """
        return self._neutral_sheet


    @neutral_sheet.setter
    def neutral_sheet(self, value):
        """
        Sets the neutral_sheet value.

        Parameters
        ----------
        value
            neutral_sheet value.
        """
        self._neutral_sheet = value


    @property
    def bow_shock(self):
        """
        Gets the bow_shock value.

        Returns
        -------
        str
            bow_shock value.
        """
        return self._bow_shock


    @bow_shock.setter
    def bow_shock(self, value):
        """
        Sets the bow_shock value.

        Parameters
        ----------
        value
            bow_shock value.
        """
        self._bow_shock = value


    @property
    def m_pause(self):
        """
        Gets the m_pause value.

        Returns
        -------
        str
            m_pause value.
        """
        return self._m_pause


    @m_pause.setter
    def m_pause(self, value):
        """
        Sets the m_pause value.

        Parameters
        ----------
        value
            m_pause value.
        """
        self._m_pause = value


    @property
    def b_gse_xyz(self):
        """
        Gets the b_gse_xyz value.

        Returns
        -------
        str
            b_gse_xyz value.
        """
        return self._b_gse_xyz


    @b_gse_xyz.setter
    def b_gse_xyz(self, value):
        """
        Sets the b_gse_xyz value.

        Parameters
        ----------
        value
            b_gse_xyz value.
        """
        self._b_gse_xyz = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('DistanceFromOptions', {})
        builder.start('NeutralSheet', {})
        builder.data(str(self._neutral_sheet).lower())
        builder.end('NeutralSheet')
        builder.start('BowShock', {})
        builder.data(str(self._bow_shock).lower())
        builder.end('BowShock')
        builder.start('MPause', {})
        builder.data(str(self._m_pause).lower())
        builder.end('MPause')
        builder.start('BGseXYZ', {})
        builder.data(str(self._b_gse_xyz).lower())
        builder.end('BGseXYZ')
        builder.end('DistanceFromOptions')

        return builder.close()


class BFieldTraceOptions:
    """
    Class representing a BFieldTraceOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    coordinate_system
        Specifies the coordinate system.
    hemisphere
        Specifies the hemisphere.
    footpoint_latitude
        Specifies whether to include footpoint latitude values in the
        listing.
    footpoint_longitude
        Specifies whether to include footpoint longitude values in the
        listing.
    field_line_length
        Specifies whether to include field line length values in the
        listing.
    """
    def __init__(self,
                 coordinate_system: bool = None,
                 hemisphere: Hemisphere = None,
                 footpoint_latitude: bool = None,
                 footpoint_longitude: bool = None,
                 field_line_length: bool = None
                 ):  # pylint: disable=too-many-arguments
        """
        Creates an object representing a BFieldTraceOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        coordinate_system
            Specifies the coordinate system.  If None, default is 
            CoordinateSystem.GEO.
        hemisphere
            Specifies the hemisphere.  If None, default is Hemisphere.NORTH.
        footpoint_latitude
            Specifies whether to include footpoint latitude values in the
            listing.
        footpoint_longitude
            Specifies whether to include footpoint longitude values in the
            listing.
        field_line_length
            Specifies whether to include field line length values in the
            listing.
        """

        if coordinate_system is None:
            self._coordinate_system = CoordinateSystem.GEO
        else:
            self._coordinate_system = coordinate_system

        if hemisphere is None:
            self._hemisphere = Hemisphere.NORTH
        else:
            self._hemisphere = hemisphere

        if footpoint_latitude is None:
            self._footpoint_latitude = False
        else:
            self._footpoint_latitude = footpoint_latitude

        if footpoint_longitude is None:
            self._footpoint_longitude = False
        else:
            self._footpoint_longitude = footpoint_longitude

        if field_line_length is None:
            self._field_line_length = False
        else:
            self._field_line_length = field_line_length


    @property
    def coordinate_system(self):
        """
        Gets the coordinate_system value.

        Returns
        -------
        str
            coordinate_system value.
        """
        return self._coordinate_system


    @coordinate_system.setter
    def coordinate_system(self, value):
        """
        Sets the coordinate_system value.

        Parameters
        ----------
        value
            coordinate_system value.
        """
        self._coordinate_system = value


    @property
    def hemisphere(self):
        """
        Gets the hemisphere value.

        Returns
        -------
        str
            hemisphere value.
        """
        return self._hemisphere


    @hemisphere.setter
    def hemisphere(self, value):
        """
        Sets the hemisphere value.

        Parameters
        ----------
        value
            hemisphere value.
        """
        self._hemisphere = value


    @property
    def footpoint_latitude(self):
        """
        Gets the footpoint_latitude value.

        Returns
        -------
        str
            footpoint_latitude value.
        """
        return self._footpoint_latitude


    @footpoint_latitude.setter
    def footpoint_latitude(self, value):
        """
        Sets the footpoint_latitude value.

        Parameters
        ----------
        value
            footpoint_latitude value.
        """
        self._footpoint_latitude = value


    @property
    def footpoint_longitude(self):
        """
        Gets the footpoint_longitude value.

        Returns
        -------
        str
            footpoint_longitude value.
        """
        return self._footpoint_longitude


    @footpoint_longitude.setter
    def footpoint_longitude(self, value):
        """
        Sets the footpoint_longitude value.

        Parameters
        ----------
        value
            footpoint_longitude value.
        """
        self._footpoint_longitude = value


    @property
    def field_line_length(self):
        """
        Gets the field_line_length value.

        Returns
        -------
        str
            field_line_length value.
        """
        return self._field_line_length


    @field_line_length.setter
    def field_line_length(self, value):
        """
        Sets the field_line_length value.

        Parameters
        ----------
        value
            field_line_length value.
        """
        self._field_line_length = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('BFieldTraceOptions', {})
        builder.start('CoordinateSystem', {})
        builder.data(self._coordinate_system.value)
        builder.end('CoordinateSystem')
        builder.start('Hemisphere', {})
        builder.data(self._hemisphere.value)
        builder.end('Hemisphere')
        builder.start('FootpointLatitude', {})
        builder.data(str(self._footpoint_latitude).lower())
        builder.end('FootpointLatitude')
        builder.start('FootpointLongitude', {})
        builder.data(str(self._footpoint_longitude).lower())
        builder.end('FootpointLongitude')
        builder.start('FieldLineLength', {})
        builder.data(str(self._field_line_length).lower())
        builder.end('FieldLineLength')
        builder.end('BFieldTraceOptions')

        return builder.close()


class OutputOptions:
    """
    Class representing a OutputOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    coordinate_options
        Coordinate options.
    all_location_filters
        All location filters flag.
    min_max_points
        Minimum/Maximum number of points.
    region_options
        Region options.
    value_options
        Value options.
    distance_from_options
        Distance-from options.
    b_field_trace_options
        B-field trace options.
    """
    def __init__(self,
                 coordinate_options: List[FilteredCoordinateOptions],
                 all_location_filters: bool = None,
                 min_max_points: int = None,
                 region_options: RegionOptions = None,
                 value_options: ValueOptions = None,
                 distance_from_options: DistanceFromOptions = None,
                 b_field_trace_options: List[BFieldTraceOptions] = None
                 ):  # pylint: disable=too-many-arguments
        """
        Creates an object representing a OutputOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        coordinate_options
            Coordinate options.
        all_location_filters
            All location filters flag.  If None, default is True.
        min_max_points
            Minimum/Maximum number of points.  If None, default is 2.
        region_options
            Region options.
        value_options
            Value options.
        distance_from_options
            Distance-from options.
        b_field_trace_options
            B-field trace options.
        """

        self._coordinate_options = coordinate_options
        if all_location_filters is None:
            self._all_location_filters = True
        else:
            self._all_location_filters = all_location_filters
        if min_max_points is None:
            self._min_max_points = 2
        else:
            self._min_max_points = min_max_points
        self._region_options = region_options
        self._value_options = value_options
        self._distance_from_options = distance_from_options

        if b_field_trace_options is None:
            self._b_field_trace_options = []
        else:
            self._b_field_trace_options = b_field_trace_options


    @property
    def coordinate_options(self):
        """
        Gets the coordinate_options value.

        Returns
        -------
        str
            coordinate_options value.
        """
        return self._coordinate_options


    @coordinate_options.setter
    def coordinate_options(self, value):
        """
        Sets the coordinate_options value.

        Parameters
        ----------
        value
            coordinate_options value.
        """
        self._coordinate_options = value


    @property
    def all_location_filters(self):
        """
        Gets the all_location_filters value.

        Returns
        -------
        str
            all_location_filters value.
        """
        return self._all_location_filters


    @all_location_filters.setter
    def all_location_filters(self, value):
        """
        Sets the all_location_filters value.

        Parameters
        ----------
        value
            all_location_filters value.
        """
        self._all_location_filters = value


    @property
    def min_max_points(self):
        """
        Gets the min_max_points value.

        Returns
        -------
        str
            min_max_points value.
        """
        return self._min_max_points


    @min_max_points.setter
    def min_max_points(self, value):
        """
        Sets the min_max_points value.

        Parameters
        ----------
        value
            min_max_points value.
        """
        self._min_max_points = value


    @property
    def region_options(self):
        """
        Gets the region_options value.

        Returns
        -------
        str
            region_options value.
        """
        return self._region_options


    @region_options.setter
    def region_options(self, value):
        """
        Sets the region_options value.

        Parameters
        ----------
        value
            region_options value.
        """
        self._region_options = value


    @property
    def value_options(self):
        """
        Gets the value_options value.

        Returns
        -------
        str
            value_options value.
        """
        return self._value_options


    @value_options.setter
    def value_options(self, value):
        """
        Sets the value_options value.

        Parameters
        ----------
        value
            value_options value.
        """
        self._value_options = value


    @property
    def distance_from_options(self):
        """
        Gets the distance_from_options value.

        Returns
        -------
        str
            distance_from_options value.
        """
        return self._distance_from_options


    @distance_from_options.setter
    def distance_from_options(self, value):
        """
        Sets the distance_from_options value.

        Parameters
        ----------
        value
            distance_from_options value.
        """
        self._distance_from_options = value


    @property
    def b_field_trace_options(self):
        """
        Gets the b_field_trace_options value.

        Returns
        -------
        str
            b_field_trace_options value.
        """
        return self._b_field_trace_options


    @b_field_trace_options.setter
    def b_field_trace_options(self, value):
        """
        Sets the b_field_trace_options value.

        Parameters
        ----------
        value
            b_field_trace_options value.
        """
        self._b_field_trace_options = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        builder = ET.TreeBuilder()
        builder.start('OutputOptions', {})
        builder.start('AllLocationFilters', {})
        builder.data(str(self._all_location_filters).lower())
        builder.end('AllLocationFilters')
        builder.start('MinMaxPoints', {})
        builder.data(str(self._min_max_points).lower())
        builder.end('MinMaxPoints')
        builder.end('OutputOptions')

        xml_element = builder.close()

        for coord_option in self._coordinate_options:
            xml_element.append(coord_option.xml_element())
        if self._region_options is not None:
            xml_element.append(self._region_options.xml_element())
        if self._value_options is not None:
            xml_element.append(self._value_options.xml_element())
        if self._distance_from_options is not None:
            xml_element.append(self._distance_from_options.xml_element())

        for b_trace_option in self._b_field_trace_options:
            xml_element.append(b_trace_option.xml_element())

        return xml_element



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent the Request and its
sub-classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import xml.etree.ElementTree as ET

from sscws.outputoptions import LocationFilter



class RegionFilterOptions:
    """
    Class representing a RegionFilterOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    space_regions
        Specifies the space region filter options.
    radial_trace_regions
        Specifies the radial trace region filter options.
    magnetic_trace_regions
        Specifies the magnetic trace region filter options.
    """
    def __init__(self,
                 space_regions: bool = None,
                 radial_trace_regions: bool = None,
                 magnetic_trace_regions: bool = None):
        """
        Creates an object representing a RegionFilterOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        space_regions
            Specifies the space region filter options.
        radial_trace_regions
            Specifies the radial trace region filter options.
        magnetic_trace_regions
            Specifies the magnetic trace region filter options.
        """

        self._space_regions = space_regions
        self._radial_trace_regions = radial_trace_regions
        self._magnetic_trace_regions = magnetic_trace_regions


    @property
    def space_regions(self):
        """
        Gets the space_regions value.

        Returns
        -------
        SpaceRegionsFilterOptions
            space_regions value.
        """
        return self._space_regions


    @space_regions.setter
    def space_regions(self, value):
        """
        Sets the space_regions value.

        Parameters
        ----------
        value
            space_regions value.
        """
        self._space_regions = value


    @property
    def radial_trace_regions(self):
        """
        Gets the radial_trace_regions value.

        Returns
        -------
        MappedRegionFilterOptions
            radial_trace_regions value.
        """
        return self._radial_trace_regions


    @radial_trace_regions.setter
    def radial_trace_regions(self, value):
        """
        Sets the radial_trace_regions value.

        Parameters
        ----------
        value
            radial_trace_regions value.
        """
        self._radial_trace_regions = value


    @property
    def magnetic_trace_regions(self):
        """
        Gets the magnetic_trace_regions value.

        Returns
        -------
        MappedRegionFilterOptions
            magnetic_trace_regions value.
        """
        return self._magnetic_trace_regions


    @magnetic_trace_regions.setter
    def magnetic_trace_regions(self, value):
        """
        Sets the magnetic_trace_regions value.

        Parameters
        ----------
        value
            magnetic_trace_regions value.
        """
        self._magnetic_trace_regions = value



    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """
        builder = ET.TreeBuilder()

        builder.start('RegionFilterOptions', {})
        builder.end('RegionFilterOptions')

        xml_element = builder.close()

        if self._space_regions is not None:
            xml_element.append(
                self._space_regions.xml_element())
        if self._radial_trace_regions is not None:
            xml_element.append(
                self._radial_trace_regions.xml_element(
                    'RadialTraceRegions'))
        if self._magnetic_trace_regions is not None:
            xml_element.append(
                self._magnetic_trace_regions.xml_element(
                    'MagneticTraceRegions'))

        return xml_element


class LocationFilterOptions:  # pylint: disable=too-many-instance-attributes
    """
    Class representing a LocationFilterOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    all_filters
        Coordinate options.
    distance_from_center_of_earth
        All location filters flag.
    magnetic_field_strength
        Minimum/Maximum number of points.
    distance_from_neutral_sheet
        Region options.
    distance_from_bow_shock
        Value options.
    distance_from_magnetopause
        Distance-from magnetopause options.
    dipole_l_value
        Dipole L value options.
    dipole_invariant_latitude
        Dipole invariant latitude options.
    """
    def __init__(self,
                 all_filters: bool = None,
                 distance_from_center_of_earth: LocationFilter = None,
                 magnetic_field_strength: LocationFilter = None,
                 distance_from_neutral_sheet: LocationFilter = None,
                 distance_from_bow_shock: LocationFilter = None,
                 distance_from_magnetopause: LocationFilter = None,
                 dipole_l_value: LocationFilter = None,
                 dipole_invariant_latitude: LocationFilter = None
                 ):  # pylint: disable=too-many-arguments
        """
        Creates an object representing a LocationFilterOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        all_filters
            Coordinate options.
        distance_from_center_of_earth
            All location filters flag.
        magnetic_field_strength
            Minimum/Maximum number of points.
        distance_from_neutral_sheet
            Region options.
        distance_from_bow_shock
            Value options.
        distance_from_magnetopause
            Distance-from magnetopause options.
        dipole_l_value
            Dipole L value options.
        dipole_invariant_latitude
            Dipole invariant latitude options.
        """

        if all_filters is None:
            self._all_filters = True
        else:
            self._all_filters = all_filters

        if distance_from_center_of_earth is not None:
            self._distance_from_center_of_earth = \
                distance_from_center_of_earth

        if magnetic_field_strength is not None:
            self._magnetic_field_strength = magnetic_field_strength

        if distance_from_neutral_sheet is not None:
            self._distance_from_neutral_sheet = distance_from_neutral_sheet

        if distance_from_bow_shock is not None:
            self._distance_from_bow_shock = distance_from_bow_shock

        if distance_from_magnetopause is not None:
            self._distance_from_magnetopause = distance_from_magnetopause

        if dipole_l_value is not None:
            self._dipole_l_value = dipole_l_value

        if dipole_invariant_latitude is not None:
            self._dipole_invariant_latitude = dipole_invariant_latitude


    @property
    def all_filters(self):
        """
        Gets the all_filters value.

        Returns
        -------
        str
            all_filters value.
        """
        return self._all_filters


    @all_filters.setter
    def all_filters(self, value):
        """
        Sets the all_filters value.

        Parameters
        ----------
        value
            all_filters value.
        """
        self._all_filters = value


    @property
    def distance_from_center_of_earth(self):
        """
        Gets the distance_from_center_of_earth value.

        Returns
        -------
        str
            distance_from_center_of_earth value.
        """
        return self._distance_from_center_of_earth


    @distance_from_center_of_earth.setter
    def distance_from_center_of_earth(self, value):
        """
        Sets the distance_from_center_of_earth value.

        Parameters
        ----------
        value
            distance_from_center_of_earth value.
        """
        self._distance_from_center_of_earth = value


    @property
    def magnetic_field_strength(self):
        """
        Gets the magnetic_field_strength value.

        Returns
        -------
        str
            magnetic_field_strength value.
        """
        return self._magnetic_field_strength


    @magnetic_field_strength.setter
    def magnetic_field_strength(self, value):
        """
        Sets the magnetic_field_strength value.

        Parameters
        ----------
        value
            magnetic_field_strength value.
        """
        self._magnetic_field_strength = value


    @property
    def distance_from_neutral_sheet(self):
        """
        Gets the distance_from_neutral_sheet value.

        Returns
        -------
        str
            distance_from_neutral_sheet value.
        """
        return self._distance_from_neutral_sheet


    @distance_from_neutral_sheet.setter
    def distance_from_neutral_sheet(self, value):
        """
        Sets the distance_from_neutral_sheet value.

        Parameters
        ----------
        value
            distance_from_neutral_sheet value.
        """
        self._distance_from_neutral_sheet = value


    @property
    def distance_from_bow_shock(self):
        """
        Gets the distance_from_bow_shock value.

        Returns
        -------
        str
            distance_from_bow_shock value.
        """
        return self._distance_from_bow_shock


    @distance_from_bow_shock.setter
    def distance_from_bow_shock(self, value):
        """
        Sets the distance_from_bow_shock value.

        Parameters
        ----------
        value
            distance_from_bow_shock value.
        """
        self._distance_from_bow_shock = value


    @property
    def distance_from_magnetopause(self):
        """
        Gets the distance_from_magnetopause value.

        Returns
        -------
        str
            distance_from_magnetopause value.
        """
        return self._distance_from_magnetopause


    @distance_from_magnetopause.setter
    def distance_from_magnetopause(self, value):
        """
        Sets the distance_from_magnetopause value.

        Parameters
        ----------
        value
            distance_from_magnetopause value.
        """
        self._distance_from_magnetopause = value


    @property
    def dipole_l_value(self):
        """
        Gets the dipole_l_value value.

        Returns
        -------
        str
            dipole_l_value value.
        """
        return self._dipole_l_value


    @dipole_l_value.setter
    def dipole_l_value(self, value):
        """
        Sets the dipole_l_value value.

        Parameters
        ----------
        value
            dipole_l_value value.
        """
        self._dipole_l_value = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        builder = ET.TreeBuilder()
        builder.start('LocationFilterOptions', {})
        builder.start('AllFilters', {})
        builder.data(str(self._all_filters).lower())
        builder.end('AllFilters')
        xml_element = builder.close()

        if self._distance_from_center_of_earth is not None:
            xml_element.append(
                self._distance_from_center_of_earth.xml_element(
                    'DistanceFromCenterOfEarth'))
        if self._magnetic_field_strength is not None:
            xml_element.append(
                self._magnetic_field_strength.xml_element(
                    'MagneticFieldStrength'))
        if self._distance_from_neutral_sheet is not None:
            xml_element.append(
                self._distance_from_neutral_sheet.xml_element(
                    'DistanceFromNeutralSheet'))
        if self._distance_from_bow_shock is not None:
            xml_element.append(
                self._distance_from_bow_shock.xml_element(
                    'DistanceFromBowShock'))
        if self._distance_from_magnetopause is not None:
            xml_element.append(
                self._distance_from_magnetopause.xml_element(
                    'DistanceFromMagnetopause'))
        if self._dipole_l_value is not None:
            xml_element.append(
                self._dipole_l_value.xml_element('DipoleLValue'))
        if self._dipole_invariant_latitude is not None:
            xml_element.append(
                self._dipole_invariant_latitude.xml_element(
                    'DipoleInvariantLatitude'))

        return xml_element


class SpaceRegionsFilterOptions:  # pylint: disable=too-many-instance-attributes
    """
    Class representing a SpaceRegionsFilterOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    interplanetary_medium
        Specifies whether the interplanetary medium region filter is
        to be applied.  If None, default is False.
    dayside_magnetosheath
        Specifies whether the dayside magnetosheath region filter is
        to be applied.  If None, default is False.
    nightside_magnetosheath
        Specifies whether the nightside magnetosheath region filter is
        to be applied.  If None, default is False.
    dayside_magnetosphere
        Specifies whether the dayside magnetosphere region filter is
        to be applied.  If None, default is False.
    nightside_magnetosphere
        Specifies whether the nightside magnetosphere region filter is
        to be applied.  If None, default is False.
    plasma_sheet
        Specifies whether the plasma sheet region filter is to be
        applied.  If None, default is False.
    tail_lobe
        Specifies whether the tail lobe region filter is to be applied.
        If None, default is False.
    high_latitude_boundary_layer
        Specifies whether the high latitude boundary layer region
        filter is to be applied.  If None, default is False.
    low_latitude_boundary_layer
        Specifies whether the low latitude boundary layer region
        filter is to be applied.  If None, default is False.
    dayside_plasmasphere
        Specifies whether the dayside plasmasphere region
        filter is to be applied.  If None, default is False.
    nightside_plasmasphere
        Specifies whether the nightside plasmasphere region
        filter is to be applied.  If None, default is False.
    """
    def __init__(self,
                 interplanetary_medium: bool = None,
                 dayside_magnetosheath: bool = None,
                 nightside_magnetosheath: bool = None,
                 dayside_magnetosphere: bool = None,
                 nightside_magnetosphere: bool = None,
                 plasma_sheet: bool = None,
                 tail_lobe: bool = None,
                 high_latitude_boundary_layer: bool = None,
                 low_latitude_boundary_layer: bool = None,
                 dayside_plasmasphere: bool = None,
                 nightside_plasmasphere: bool = None
                 ):  # pylint: disable=too-many-arguments,too-many-branches
        """
        Creates an object representing a SpaceRegionsFilterOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        interplanetary_medium
            Specifies whether the interplanetary medium region filter is
            to be applied.  If None, default is False.
        dayside_magnetosheath
            Specifies whether the dayside magnetosheath region filter is
            to be applied.  If None, default is False.
        nightside_magnetosheath
            Specifies whether the nightside magnetosheath region filter is
            to be applied.  If None, default is False.
        dayside_magnetosphere
            Specifies whether the dayside magnetosphere region filter is
            to be applied.  If None, default is False.
        nightside_magnetosphere
            Specifies whether the nightside magnetosphere region filter is
            to be applied.  If None, default is False.
        plasma_sheet
            Specifies whether the plasma sheet region filter is to be
            applied.  If None, default is False.
        tail_lobe
            Specifies whether the tail lobe region filter is to be applied.
            If None, default is False.
        high_latitude_boundary_layer
            Specifies whether the high latitude boundary layer region
            filter is to be applied.  If None, default is False.
        low_latitude_boundary_layer
            Specifies whether the low latitude boundary layer region
            filter is to be applied.  If None, default is False.
        dayside_plasmasphere
            Specifies whether the dayside plasmasphere region
            filter is to be applied.  If None, default is False.
        nightside_plasmasphere
            Specifies whether the nightside plasmasphere region
            filter is to be applied.  If None, default is False.
        """

        if interplanetary_medium is None:
            self._interplanetary_medium = False
        else:
            self._interplanetary_medium = interplanetary_medium
        if dayside_magnetosheath is None:
            self._dayside_magnetosheath = False
        else:
            self._dayside_magnetosheath = dayside_magnetosheath
        if nightside_magnetosheath is None:
            self._nightside_magnetosheath = False
        else:
            self._nightside_magnetosheath = nightside_magnetosheath
        if dayside_magnetosphere is None:
            self._dayside_magnetosphere = False
        else:
            self._dayside_magnetosphere = dayside_magnetosphere
        if nightside_magnetosphere is None:
            self._nightside_magnetosphere = False
        else:
            self._nightside_magnetosphere = nightside_magnetosphere
        if plasma_sheet is None:
            self._plasma_sheet = False
        else:
            self._plasma_sheet = plasma_sheet
        if tail_lobe is None:
            self._tail_lobe = False
        else:
            self._tail_lobe = tail_lobe
        if high_latitude_boundary_layer is None:
            self._high_latitude_boundary_layer = False
        else:
            self._high_latitude_boundary_layer = high_latitude_boundary_layer
        if low_latitude_boundary_layer is None:
            self._low_latitude_boundary_layer = False
        else:
            self._low_latitude_boundary_layer = low_latitude_boundary_layer
        if dayside_plasmasphere is None:
            self._dayside_plasmasphere = False
        else:
            self._dayside_plasmasphere = dayside_plasmasphere
        if nightside_plasmasphere is None:
            self._nightside_plasmasphere = False
        else:
            self._nightside_plasmasphere = nightside_plasmasphere


    @property
    def interplanetary_medium(self):
        """
        Gets the interplanetary_medium value.

        Returns
        -------
        bool
            interplanetary_medium value.
        """
        return self._interplanetary_medium


    @interplanetary_medium.setter
    def interplanetary_medium(self, value):
        """
        Sets the interplanetary_medium value.

        Parameters
        ----------
        value
            interplanetary_medium value.
        """
        self._interplanetary_medium = value


    @property
    def dayside_magnetosheath(self):
        """
        Gets the dayside_magnetosheath value.

        Returns
        -------
        bool
            dayside_magnetosheath value.
        """
        return self._dayside_magnetosheath


    @dayside_magnetosheath.setter
    def dayside_magnetosheath(self, value):
        """
        Sets the dayside_magnetosheath value.

        Parameters
        ----------
        value
            dayside_magnetosheath value.
        """
        self._dayside_magnetosheath = value


    @property
    def nightside_magnetosheath(self):
        """
        Gets the nightside_magnetosheath value.

        Returns
        -------
        bool
            nightside_magnetosheath value.
        """
        return self._nightside_magnetosheath


    @nightside_magnetosheath.setter
    def nightside_magnetosheath(self, value):
        """
        Sets the nightside_magnetosheath value.

        Parameters
        ----------
        value
            nightside_magnetosheath value.
        """
        self._nightside_magnetosheath = value


    @property
    def dayside_magnetosphere(self):
        """
        Gets the dayside_magnetosphere value.

        Returns
        -------
        bool
            dayside_magnetosphere value.
        """
        return self._dayside_magnetosphere


    @dayside_magnetosphere.setter
    def dayside_magnetosphere(self, value):
        """
        Sets the dayside_magnetosphere value.

        Parameters
        ----------
        value
            dayside_magnetosphere value.
        """
        self._dayside_magnetosphere = value


    @property
    def nightside_magnetosphere(self):
        """
        Gets the nightside_magnetosphere value.

        Returns
        -------
        bool
            nightside_magnetosphere value.
        """
        return self._nightside_magnetosphere


    @nightside_magnetosphere.setter
    def nightside_magnetosphere(self, value):
        """
        Sets the nightside_magnetosphere value.

        Parameters
        ----------
        value
            nightside_magnetosphere value.
        """
        self._nightside_magnetosphere = value


    @property
    def plasma_sheet(self):
        """
        Gets the plasma_sheet value.

        Returns
        -------
        bool
            plasma_sheet value.
        """
        return self._plasma_sheet


    @plasma_sheet.setter
    def plasma_sheet(self, value):
        """
        Sets the plasma_sheet value.

        Parameters
        ----------
        value
            plasma_sheet value.
        """
        self._plasma_sheet = value


    @property
    def tail_lobe(self):
        """
        Gets the tail_lobe value.

        Returns
        -------
        bool
            tail_lobe value.
        """
        return self._tail_lobe


    @tail_lobe.setter
    def tail_lobe(self, value):
        """
        Sets the tail_lobe value.

        Parameters
        ----------
        value
            tail_lobe value.
        """
        self._tail_lobe = value


    @property
    def high_latitude_boundary_layer(self):
        """
        Gets the high_latitude_boundary_layer value.

        Returns
        -------
        bool
            high_latitude_boundary_layer value.
        """
        return self._high_latitude_boundary_layer


    @high_latitude_boundary_layer.setter
    def high_latitude_boundary_layer(self, value):
        """
        Sets the high_latitude_boundary_layer value.

        Parameters
        ----------
        value
            high_latitude_boundary_layer value.
        """
        self._high_latitude_boundary_layer = value


    @property
    def low_latitude_boundary_layer(self):
        """
        Gets the low_latitude_boundary_layer value.

        Returns
        -------
        bool
            low_latitude_boundary_layer value.
        """
        return self._low_latitude_boundary_layer


    @low_latitude_boundary_layer.setter
    def low_latitude_boundary_layer(self, value):
        """
        Sets the low_latitude_boundary_layer value.

        Parameters
        ----------
        value
            low_latitude_boundary_layer value.
        """
        self._low_latitude_boundary_layer = value


    @property
    def dayside_plasmasphere(self):
        """
        Gets the dayside_plasmasphere value.

        Returns
        -------
        bool
            dayside_plasmasphere value.
        """
        return self._dayside_plasmasphere


    @dayside_plasmasphere.setter
    def dayside_plasmasphere(self, value):
        """
        Sets the dayside_plasmasphere value.

        Parameters
        ----------
        value
            dayside_plasmasphere value.
        """
        self._dayside_plasmasphere = value


    @property
    def nightside_plasmasphere(self):
        """
        Gets the nightside_plasmasphere value.

        Returns
        -------
        bool
            nightside_plasmasphere value.
        """
        return self._nightside_plasmasphere


    @nightside_plasmasphere.setter
    def nightside_plasmasphere(self, value):
        """
        Sets the nightside_plasmasphere value.

        Parameters
        ----------
        value
            nightside_plasmasphere value.
        """
        self._nightside_plasmasphere = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        builder = ET.TreeBuilder()
        builder.start('SpaceRegions', {})
        builder.start('InterplanetaryMedium', {})
        builder.data(str(self._interplanetary_medium).lower())
        builder.end('InterplanetaryMedium')

        builder.start('DaysideMagnetosheath', {})
        builder.data(str(self._dayside_magnetosheath).lower())
        builder.end('DaysideMagnetosheath')

        builder.start('NightsideMagnetosheath', {})
        builder.data(str(self._nightside_magnetosheath).lower())
        builder.end('NightsideMagnetosheath')

        builder.start('DaysideMagnetosphere', {})
        builder.data(str(self._dayside_magnetosphere).lower())
        builder.end('DaysideMagnetosphere')

        builder.start('NightsideMagnetosphere', {})
        builder.data(str(self._nightside_magnetosphere).lower())
        builder.end('NightsideMagnetosphere')

        builder.start('PlasmaSheet', {})
        builder.data(str(self._plasma_sheet).lower())
        builder.end('PlasmaSheet')

        builder.start('TailLobe', {})
        builder.data(str(self._tail_lobe).lower())
        builder.end('TailLobe')

        builder.start('HighLatitudeBoundaryLayer', {})
        builder.data(str(self._high_latitude_boundary_layer).lower())
        builder.end('HighLatitudeBoundaryLayer')

        builder.start('LowLatitudeBoundaryLayer', {})
        builder.data(str(self._low_latitude_boundary_layer).lower())
        builder.end('LowLatitudeBoundaryLayer')

        builder.start('DaysidePlasmasphere', {})
        builder.data(str(self._dayside_plasmasphere).lower())
        builder.end('DaysidePlasmasphere')

        builder.start('NightsidePlasmasphere', {})
        builder.data(str(self._nightside_plasmasphere).lower())
        builder.end('NightsidePlasmasphere')
        builder.end('SpaceRegions')

        return builder.close()


class MappedRegionFilterOptions:
    """
    Class representing a MappedRegionFilterOptions from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    cusp
        Cusp region.
    cleft
        Cleft region.
    auroral_oval
        Auroral Oval region.
    polar_cap
        Polar Cap region.
    mid_latitude
        Mid-Latitude region.
    low_latitude
        Low-Latitude region.
    """
    def __init__(self,
                 cusp: LocationFilter = None,
                 cleft: LocationFilter = None,
                 auroral_oval: LocationFilter = None,
                 polar_cap: LocationFilter = None,
                 mid_latitude: LocationFilter = None,
                 low_latitude: bool = None,
                 ):  # pylint: disable=too-many-arguments
        """
        Creates an object representing a MappedRegionFilterOptions from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        cusp
            Cusp region.
        cleft
            Cleft region.
        auroral_oval
            Auroral Oval region.
        polar_cap
            Polar Cap region.
        mid_latitude
            Mid-Latitude region.
        low_latitude
            Low-Latitude region.
        """

        self._cusp = cusp
        self._cleft = cleft
        self._auroral_oval = auroral_oval
        self._polar_cap = polar_cap
        self._mid_latitude = mid_latitude
        self._low_latitude = low_latitude


    @property
    def cusp(self):
        """
        Gets the cusp value.

        Returns
        -------
        str
            cusp value.
        """
        return self._cusp


    @cusp.setter
    def cusp(self, value):
        """
        Sets the cusp value.

        Parameters
        ----------
        value
            cusp value.
        """
        self._cusp = value


    @property
    def cleft(self):
        """
        Gets the cleft value.

        Returns
        -------
        str
            cleft value.
        """
        return self._cleft


    @cleft.setter
    def cleft(self, value):
        """
        Sets the cleft value.

        Parameters
        ----------
        value
            cleft value.
        """
        self._cleft = value


    @property
    def auroral_oval(self):
        """
        Gets the auroral_oval value.

        Returns
        -------
        str
            auroral_oval value.
        """
        return self._auroral_oval


    @auroral_oval.setter
    def auroral_oval(self, value):
        """
        Sets the auroral_oval value.

        Parameters
        ----------
        value
            auroral_oval value.
        """
        self._auroral_oval = value


    @property
    def polar_cap(self):
        """
        Gets the polar_cap value.

        Returns
        -------
        str
            polar_cap value.
        """
        return self._polar_cap


    @polar_cap.setter
    def polar_cap(self, value):
        """
        Sets the polar_cap value.

        Parameters
        ----------
        value
            polar_cap value.
        """
        self._polar_cap = value


    @property
    def mid_latitude(self):
        """
        Gets the mid_latitude value.

        Returns
        -------
        str
            mid_latitude value.
        """
        return self._mid_latitude


    @mid_latitude.setter
    def mid_latitude(self, value):
        """
        Sets the mid_latitude value.

        Parameters
        ----------
        value
            mid_latitude value.
        """
        self._mid_latitude = value


    @property
    def low_latitude(self):
        """
        Gets the low_latitude value.

        Returns
        -------
        str
            low_latitude value.
        """
        return self._low_latitude


    @low_latitude.setter
    def low_latitude(self, value):
        """
        Sets the low_latitude value.

        Parameters
        ----------
        value
            low_latitude value.
        """
        self._low_latitude = value


    def xml_element(self,
                    name: str) -> ET:
        """
        Produces the XML Element representation of this object.

        Parameters
        ----------
        name
            Name of this TraceRegion.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        builder = ET.TreeBuilder()
        builder.start(name, {})
        builder.end(name)
        xml_element = builder.close()

        if self._cusp is not None:
            xml_element.append(
                self._cusp.xml_element(
                    'Cusp'))
        if self._cleft is not None:
            xml_element.append(
                self._cleft.xml_element(
                    'Cleft'))
        if self._auroral_oval is not None:
            xml_element.append(
                self._auroral_oval.xml_element(
                    'AuroralOval'))
        if self._polar_cap is not None:
            xml_element.append(
                self._polar_cap.xml_element(
                    'PolarCap'))
        if self._mid_latitude is not None:
            xml_element.append(
                self._mid_latitude.xml_element(
                    'MidLatitude'))
        if self._low_latitude is not None:
            builder = ET.TreeBuilder()
            builder.start('LowLatitude', {})
            builder.data(str(self._low_latitude).lower())
            builder.end('LowLatitude')
            xml_element.append(builder.close())

        return xml_element



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent the BFieldModel and its
sub-classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import xml.etree.ElementTree as ET
from abc import ABCMeta
from enum import Enum



class InternalBFieldModel(Enum):
    """
    Enumerations representing the InternalBFieldModel defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    IGRF = 'IGRF'
    SIMPLE_DIPOLE = 'SimpleDipole'


class ExternalBFieldModelName(Enum):
    """
    Enumerations representing the name of ExternalBFieldModel defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    TSYGANENKO96 = 'Tsyganenko96BFieldModel'
    TSYGANENKO89C = 'Tsyganenko89cBFieldModel'
    TSYGANENKO87 = 'Tsyganenko87BFieldModel'


class ExternalBFieldModel(metaclass=ABCMeta):
    """
    Enumerations representing the ExternalBFieldModel defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    name
        Sub-type name.
    """
    def __init__(self,
                 name: ExternalBFieldModelName):
        self._name = name


    def xml_element(self):
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        builder = ET.TreeBuilder()
        builder.start('ExternalBFieldModel', {
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:type': self._name.value
        })
        builder.end('ExternalBFieldModel')
        return builder.close()


    @property
    def name(self):
        """
        Gets the name value.

        Returns
        -------
        str
            name value.
        """
        return self._name


    @name.setter
    def name(self, value):
        """
        Sets the name value.

        Parameters
        ----------
        value
            name value.
        """
        self._name = value


class Tsyganenko96BFieldModel(ExternalBFieldModel):
    """
    Class representing the Tsyganenko96BFieldModel defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    solar_wind_pressure
        Solar wind pressure (nP).
    dst_index
        Disturbance Storm Time (DST) index.
    by_imf
        BY Interplanetary Magnetic Field (IMF).
    bz_imf
        BZ Interplanetary Magnetic Field (IMF).
    """
    def __init__(self,
                 solar_wind_pressure: float = None,
                 dst_index: int = None,
                 by_imf: float = None,
                 bz_imf: float = None):
        """
        Creates an object representing the Tsyganenko96BFieldModel from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        solar_wind_pressure
            Solar wind pressure (nP).  If None, default is 2.1.
        dst_index
            Disturbance Storm Time (DST) index.  If None, default is -20.
        by_imf
            BY Interplanetary Magnetic Field (IMF).  If None, default
            is 0.0.
        bz_imf
            BZ Interplanetary Magnetic Field (IMF).  If None, default
            is 0.0.
        """
        super().__init__(ExternalBFieldModelName.TSYGANENKO96)

        if solar_wind_pressure is None:
            self._solar_wind_pressure = 2.1
        else:
            self._solar_wind_pressure = solar_wind_pressure

        if dst_index is None:
            self._dst_index = -20
        else:
            self._dst_index = dst_index

        if by_imf is None:
            self._by_imf = 0.0
        else:
            self._by_imf = by_imf

        if bz_imf is None:
            self._bz_imf = 0.0
        else:
            self._bz_imf = bz_imf


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        xml_element = super().xml_element()

        builder = ET.TreeBuilder()
        builder.start('SolarWindPressure', {})
        builder.data(str(self._solar_wind_pressure))
        builder.end('SolarWindPressure')
        xml_element.append(builder.close())

        builder = ET.TreeBuilder()
        builder.start('DstIndex', {})
        builder.data(str(self._dst_index))
        builder.end('DstIndex')
        xml_element.append(builder.close())

        builder = ET.TreeBuilder()
        builder.start('ByImf', {})
        builder.data(str(self._by_imf))
        builder.end('ByImf')
        xml_element.append(builder.close())

        builder = ET.TreeBuilder()
        builder.start('BzImf', {})
        builder.data(str(self._bz_imf))
        builder.end('BzImf')
        xml_element.append(builder.close())

        return xml_element


class Tsyganenko87Kp(Enum):
    """
    Enumerations representing the Tsyganenko87Kp defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    KP_0_0 = 'KP0_0'
    KP_1_1_1 = 'KP1_1_1'
    KP_2_2_2 = 'KP2_2_2'
    KP_3_3_3 = 'KP3_3_3'
    KP_4_4_4 = 'KP4_4_4'
    KP_5 = 'KP5'


class Tsyganenko87BFieldModel(ExternalBFieldModel):
    """
    Class representing the Tsyganenko87BFieldModel defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    def __init__(self,
                 key_parameters: Tsyganenko87Kp = None):
        """
        Creates an object representing the Tsyganenko87BFieldModel from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        key_parameters
            Model key parameter values.  If None, default is
            Tsyganenko87Kp.KP_3_3_3.
        """
        super().__init__(ExternalBFieldModelName.TSYGANENKO87)

        if key_parameters is None:
            self._key_parameters = Tsyganenko87Kp.KP_3_3_3
        else:
            self._key_parameters = key_parameters


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        xml_element = super().xml_element()

        builder = ET.TreeBuilder()
        builder.start('KeyParameterValues', {})
        builder.data(self._key_parameters.value)
        builder.end('KeyParameterValues')

        xml_element.append(builder.close())

        return xml_element


class Tsyganenko89cKp(Enum):
    """
    Enumerations representing the Tsyganenko89cKp defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    KP_0_0 = 'KP0_0'
    KP_1_1_1 = 'KP1_1_1'
    KP_2_2_2 = 'KP2_2_2'
    KP_3_3_3 = 'KP3_3_3'
    KP_4_4_4 = 'KP4_4_4'
    KP_5_5_5 = 'KP5_5_5'
    KP_6 = 'KP6'


class Tsyganenko89cBFieldModel(ExternalBFieldModel):
    """
    Class representing the Tsyganenko89cBFieldModel defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    def __init__(self,
                 key_parameters: Tsyganenko89cKp = None):
        """
        Creates an object representing the Tsyganenko89cBFieldModel from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        key_parameters
            Model key parameter values.  If None, default is
            Tsyganenko89cKp.KP_3_3_3.
        """
        super().__init__(ExternalBFieldModelName.TSYGANENKO89C)

        if key_parameters is None:
            self._key_parameters = Tsyganenko89cKp.KP_3_3_3
        else:
            self._key_parameters = key_parameters


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        xml_element = super().xml_element()

        builder = ET.TreeBuilder()
        builder.start('KeyParameterValues', {})
        builder.data(self._key_parameters.value)
        builder.end('KeyParameterValues')

        xml_element.append(builder.close())

        return xml_element


class BFieldModel:
    """
    Class representing a BFieldModel from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    internal
        Internal B field model.
    external
        External B field model.
    trace_stop_altitude
        Stop altitude for downward tracing of field lines.

    """
    def __init__(self,
                 internal: InternalBFieldModel = None,
                 external: ExternalBFieldModel = None,
                 trace_stop_altitude: int = None):
        """
        Constructs a BFieldModel object.

        Parameters
        ----------
        internal
            Internal B field model.  If None, the default is
            InternalBFieldModel.IGRF.
        external
            External B field model.  If None, the default is
            Tsyganenko 89c with KP 3-,3,3+.
        trace_stop_altitude
            Stop altitude for downward tracing of field lines.  If None,
            the default is 100km.
        """

        if internal is None:
            self._internal = InternalBFieldModel.IGRF
        else:
            self._internal = internal

        if external is None:
            self._external = Tsyganenko89cBFieldModel()
        else:
            self._external = external

        if trace_stop_altitude is None:
            self._trace_stop_altitude = 100
        else:
            self._trace_stop_altitude = trace_stop_altitude


    @property
    def internal(self):
        """
        Gets the internal value.

        Returns
        -------
        InternalBFieldModel
            internal value.
        """
        return self._internal


    @internal.setter
    def internal(self, value):
        """
        Sets the internal value.

        Parameters
        ----------
        value
            internal value.
        """
        self._internal = value


    @property
    def external(self):
        """
        Gets the external value.

        Returns
        -------
        ExternalBFieldModel
            external value.
        """
        return self._external


    @external.setter
    def external(self, value):
        """
        Sets the external value.

        Parameters
        ----------
        value
            external value.
        """
        self._external = value


    @property
    def trace_stop_altitude(self):
        """
        Gets the trace_stop_altitude value.

        Returns
        -------
        int
            trace_stop_altitude value.
        """
        return self._trace_stop_altitude


    @trace_stop_altitude.setter
    def trace_stop_altitude(self, value):
        """
        Sets the trace_stop_altitude value.

        Parameters
        ----------
        value
            trace_stop_altitude value.
        """
        self._trace_stop_altitude = value


    def xml_element(self) -> ET:
        """
        Produces the XML Element representation of this object.

        Returns
        -------
        ET
            XML Element represenation of this object.
        """

        builder = ET.TreeBuilder()
        builder.start('BFieldModel', {})
        builder.start('InternalBFieldModel', {})
        builder.data(self._internal.value)
        builder.end('InternalBFieldModel')

        builder.start('TraceStopAltitude', {})
        builder.data(str(self._trace_stop_altitude))
        builder.end('TraceStopAltitude')

        builder.end('BFieldModel')

        xml_element = builder.close()
        xml_element.append(self._external.xml_element())

        return xml_element



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Module defining classes to represent the Result class and its
sub-classes from
<https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.<br>

Copyright &copy; 2013-2020 United States Government as represented by the
National Aeronautics and Space Administration. No copyright is claimed in
the United States under Title 17, U.S.Code. All Other Rights Reserved.
"""

import xml.etree.ElementTree as ET
from typing import List
from abc import ABCMeta, abstractmethod
from enum import Enum


class ResultStatusCode(Enum):
    """
    Enumerations representing the ResultStatusCode defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    SUCCESS = 'Success'
    CONDITIONAL_SUCCESS = 'ConditionalSuccess'
    ERROR = 'Error'


class ResultStatusSubCode(Enum):
    """
    Enumerations representing the ResultStatusSubCode defined
    in <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    """
    SUCCESS = 'Success'
    MISSING_REQUEST = 'MissingRequest'
    MISSING_SATELLITES = 'MissingSatellites'
    INVALID_BEGIN_TIME = 'InvalidBeginTime'
    INVALID_END_TIME = 'InvalidEndTime'
    INVALID_SATELLITE = 'InvalidSatellite'
    INVALID_TIME_RANGE = 'InvalidTimeRange'
    INVALID_RESOLUTION_FACTOR = 'InvalidResolutionFactor'
    MISSING_OUTPUT_OPTIONS = 'MissingOutputOptions'
    MISSING_COORD_OPTIONS = 'MissingCoordOptions'
    MISSING_COORD_SYSTEM = 'MissingCoordSystem'
    INVALID_COORD_SYSTEM = 'InvalidCoordSystem'
    MISSING_COORD_COMPONENT = 'MissingCoordComponent'
    MISSING_GRAPH_OPTIONS = 'MissingGraphOptions'
    MISSING_COORDINATE_SYSTEM = 'MissingCoordinateSystem'
    MISSING_COORDINATE_COMPONENT = 'MissingCoordinateComponent'
    SERVER_ERROR = 'ServerError'


class Result(metaclass=ABCMeta):
    """
    Class representing a Result from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Notes
    -----
    Although this class is essentially a dictionary, it was defined as a
    class to make certain that it matched the structure and key names
    of a Request from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
    It also needs to function as a base class for the concrete
    sub-classes of a Request.

    Properties
    ----------
    status_code
        Result status code.
    status_sub_code
        Result status sub-code.
    status_text
        Result status text.
    """
    @abstractmethod
    def __init__(
            self,
            status_code: ResultStatusCode,
            status_sub_code: ResultStatusSubCode,
            status_text: List[str]):
        """
        Creates an object representing a Result from
        <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Parameters
        ----------
        status_code
        status_sub_code
        status_text
        """
        self._status_code = status_code
        self._status_sub_code = status_sub_code
        self._status_text = status_text


    @property
    def status_code(self):
        """
        Gets the status_code value.

        Returns
        -------
        str
            status_code value.
        """
        return self._status_code


    @status_code.setter
    def status_code(self, value):
        """
        Sets the status_code value.

        Parameters
        ----------
        value
            status_code value.
        """
        self._status_code = value



    @property
    def status_sub_code(self):
        """
        Gets the status_sub_code value.

        Returns
        -------
        str
            status_sub_code value.
        """
        return self._status_sub_code


    @status_sub_code.setter
    def status_sub_code(self, value):
        """
        Sets the status_sub_code value.

        Parameters
        ----------
        value
            status_sub_code value.
        """
        self._status_sub_code = value



    @property
    def status_text(self):
        """
        Gets the status_text value.

        Returns
        -------
        str
            status_text value.
        """
        return self._status_text


    @status_text.setter
    def status_text(self, value):
        """
        Sets the status_text value.

        Parameters
        ----------
        value
            status_text value.
        """
        self._status_text = value

    @staticmethod
    def get_result(
            xml: str
        ) -> Result:
        """
        Produces a Result from the given xml representation of a Result.

        Parameters
        ----------
        xml
            XML representation of a Result.
        Returns
        -------
        Result
            Result representation of given xml.
        Raises
        ------
        ValueError
            If the given xml is not a valid XML representation of Result.
        """
        result = ET.fromstring(xml)

        self._status_code = int(result.find(\
            '{http://sscweb.gsfc.nasa.gov/schema}StatusCode').text)
        self._status_sub_code = int(result.find(\
            '{http://sscweb.gsfc.nasa.gov/schema}StatusSubCode').text)

        for text in result.findall(\
            '{http://sscweb.gsfc.nasa.gov/schema}StatusText'):
            self._status_text.append(text.text)


class FileResult(Result):
    """
    Class representing a FileResult from
    <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

    Properties
    ----------
    files
        References to the files containing the requested data.
    """



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2020 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Package for accessing the Satellite Situation Center (SSC) web services
https://sscweb.gsfc.nasa.gov/WebServices/REST/.
"""

#import os
import platform
import xml.etree.ElementTree as ET
import logging
from typing import Dict, List, Tuple
import requests
import dateutil.parser

from sscws import __version__
from sscws.request import DataRequest
from sscws.coordinates import CoordinateSystem
from sscws.regions import FootpointRegion, Hemisphere, SpaceRegion



class SscWs:
    """
    Class representing the web service interface to NASA's
    Satelite Situation Center (SSC) <https://sscweb.gsfc.nasa.gov/>.

    Notes
    -----
    The logger used by this class has the class' name (SscWs).  By default,
    it is configured with a NullHandler.  Users of this class may configure
    the logger to aid in diagnosing problems.

    This class is dependent upon xml.etree.ElementTree which makes it
    vulnerable to an "exponential entity expansion" and "quadratic blowup
    entity expansion" XML attack.  But it only receives XML from the
    (trusted) SSC server.  See xml.etree.ElementTree "XML vulnerabilities"
    documentation for more details.
    """
    # pylint: disable=too-many-instance-attributes
    # pylint: disable=too-many-arguments
    def __init__(
            self,
            endpoint=None,
            timeout=None,
            proxy=None,
            ca_certs=None,
            disable_ssl_certificate_validation=False):
        """
        Creates an object representing the SSC web services.

        Parameters
        ----------
        endpoint
            URL of the SSC web service.  If None, the default is
            'https://sscweb.gsfc.nasa.gov/WS/sscr/2/'.
        timeout
            Number of seconds to wait for a response from the server.
        proxy
            HTTP proxy information.  For example,
            proxies = {
              'http': 'http://10.10.1.10:3128',
              'https': 'http://10.10.1.10:1080',
            }
            Proxy information can also be set with environment variables.
            For example,
            $ export HTTP_PROXY="http://10.10.1.10:3128"
            $ export HTTPS_PROXY="http://10.10.1.10:1080"
        ca_certs
            Path to certificate authority (CA) certificates that will
            override the default bundle.
        disable_ssl_certificate_validation
            Flag indicating whether to validate the SSL certificate.
        """

        self.logger = logging.getLogger(type(self).__name__)
        self.logger.addHandler(logging.NullHandler())

        self.retry_after_time = None

        self.logger.debug('endpoint = %s', endpoint)
        self.logger.debug('ca_certs = %s', ca_certs)
        self.logger.debug('disable_ssl_certificate_validation = %s',
                          disable_ssl_certificate_validation)

        if endpoint is None:
            self._endpoint = 'https://sscweb.gsfc.nasa.gov/WS/sscr/2/'
        else:
            self._endpoint = endpoint
        self._user_agent = 'sscws/' + __version__ + ' (' + \
            platform.python_implementation() + ' ' \
            + platform.python_version() + '; '+ platform.platform() + ')'
        self._request_headers = {
            'Content-Type' : 'application/xml',
            'Accept' : 'application/xml',
            'User-Agent' : self._user_agent
        }
        self._session = requests.Session()
        self._session.headers.update(self._request_headers)

        if ca_certs is not None:
            self._session.verify = ca_certs

        if disable_ssl_certificate_validation is True:
            self._session.verify = False

        if proxy is not None:
            self._proxy = proxy

        self._timeout = timeout

    # pylint: enable=too-many-arguments

    def __del__(self):
        """
        Destructor.  Closes all network connections.
        """

        self.close()


    def close(self) -> None:
        """
        Closes any persistent network connections.  Generally, deleting
        this object is sufficient and calling this method is unnecessary.
        """
        self._session.close()


    def get_observatories(
            self
        ) -> List[Dict]:
        """
        Gets a description of the available SSC observatories.

        Returns
        -------
        List
            An array of ObservatoryDescription dictionaries where the
            structure of the dictionary mirrors ObservatoryDescription in
            <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
        """
        url = self._endpoint + 'observatories'

        self.logger.debug('request url = %s', url)

        response = self._session.get(url, timeout=self._timeout)

        if response.status_code != 200:

            self.logger.info('%s failed with http code %d', url,
                             response.status_code)
            self.logger.info('response.text: %s', response.text)
            return []

        observatory_response = ET.fromstring(response.text)

        observatories = []

        for observatory in observatory_response.findall(\
                '{http://sscweb.gsfc.nasa.gov/schema}Observatory'):

            observatories.append({
                'Id': observatory.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Id').text,
                'Name': observatory.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Name').text,
                'Resolution': int(observatory.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Resolution').text),
                'StartTime': dateutil.parser.parse(observatory.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}StartTime').text),
                'EndTime': dateutil.parser.parse(observatory.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}EndTime').text),
                'ResourceId': observatory.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}ResourceId').text
            })

        return observatories


    def get_ground_stations(
            self
        ) -> List[Dict]:
        """
        Gets a description of the available SSC ground stations.

        Returns
        -------
        List
            An array of GroundStations dictionaries where the
            structure of the dictionary mirrors GroundStations in
            <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
        """
        url = self._endpoint + 'groundStations'

        self.logger.debug('request url = %s', url)

        response = self._session.get(url, timeout=self._timeout)

        if response.status_code != 200:

            self.logger.info('%s failed with http code %d', url,
                             response.status_code)
            self.logger.info('response.text: %s', response.text)
            return []

        ground_station_response = ET.fromstring(response.text)

        ground_stations = []

        for ground_station in ground_station_response.findall(\
                '{http://sscweb.gsfc.nasa.gov/schema}GroundStation'):

            location = ground_station.find(\
                '{http://sscweb.gsfc.nasa.gov/schema}Location')
            latitude = float(location.find(\
                '{http://sscweb.gsfc.nasa.gov/schema}Latitude').text)
            longitude = float(location.find(\
                '{http://sscweb.gsfc.nasa.gov/schema}Longitude').text)

            ground_stations.append({
                'Id': ground_station.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Id').text,
                'Name': ground_station.find(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Name').text,
                'Location': {
                    'Latitude': latitude,
                    'Longitude': longitude
                }
            })

        return ground_stations


    def get_locations(
            self,
            request: DataRequest
        ) -> Tuple[int, Dict]:
        """
        Gets the given locations DataRequest.

        Parameters
        ----------
        request
            A locations DataRequest.
        Returns
        -------
        Dict
            Dictionary whose struture mirrors Result from
            <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
        """
        url = self._endpoint + 'locations'

        self.logger.debug('request url = %s', url)

        xml_data_request = request.xml_element()

        response = self._session.post(url,
                                      data=ET.tostring(xml_data_request),
                                      timeout=self._timeout)
        if response.status_code != 200:

            try:
                # requires version 3.9
                ET.indent(xml_data_request)
            except AttributeError:
                pass
            self.logger.debug('request XML = %s', ET.tostring(xml_data_request))

            self.logger.info('%s failed with http code %d', url,
                             response.status_code)
            self.logger.info('response.text: %s', response.text)
            return (response.status_code, None)

        result_element = ET.fromstring(response.text).find(\
                             '{http://sscweb.gsfc.nasa.gov/schema}Result')

        return (response.status_code, self.__get_result(result_element))


    # pylint: disable=too-many-locals
    # pylint: disable=too-many-branches
    def __get_result(
            self,
            result_element: ET
        ) -> Dict:
        """
        Creates a dict representation of a DataResult from an ElementTree
        representation.

        Parameters
        ----------
        result_element
            ElementTree representation of a DataResult from
            <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.

        Returns
        -------
        Dict
            Dict representation of the given ElementTree DataResult
            as described in
            <https://sscweb.gsfc.nasa.gov/WebServices/REST/SSC.xsd>.
        """

        #try:
        #    # requires version 3.9
        #    ET.indent(result_element)
        #except AttributeError:
        #    pass
        #self.logger.debug('result_element XML = %s',
        #                  ET.tostring(result_element))

        result = {
            'StatusCode': result_element.find(\
                   '{http://sscweb.gsfc.nasa.gov/schema}StatusCode').text,
            'StatusSubCode': result_element.find(\
                   '{http://sscweb.gsfc.nasa.gov/schema}StatusSubCode').text,
            'Data': []
        }

        data_i = -1

        for data_element in result_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}Data'):

            data_i += 1

            coords_element = data_element.find(\
                   '{http://sscweb.gsfc.nasa.gov/schema}Coordinates')

            if coords_element is None:
                # Is this the correct result for this case???
                result['Data'].append({
                    'Id': data_element.find(\
                              '{http://sscweb.gsfc.nasa.gov/schema}Id').text
                })
                continue

            coordinates = {
                'CoordinateSystem': CoordinateSystem(coords_element.find(\
                   '{http://sscweb.gsfc.nasa.gov/schema}CoordinateSystem').text),
                'X': [],
                'Y': [],
                'Z': [],
                'Latitude': [],
                'Longitude': [],
                'LocalTime': []
            }
            result['Data'].append({
                'Id': data_element.find(\
                          '{http://sscweb.gsfc.nasa.gov/schema}Id').text,
                'Coordinates': coordinates,
                'Time': [],
                'BTraceData': [],
                'RadialLength': [],
                'MagneticStrength': [],
                'NeutralSheetDistance': [],
                'BowShockDistance': [],
                'MagnetoPauseDistance': [],
                'DipoleLValue': [],
                'DipoleInvariantLatitude': [],
                'SpacecraftRegion': [],
                'RadialTracedFootpointRegions': [],
                'BGseX': [],
                'BGseY': [],
                'BGseZ': [],
                'NorthBTracedFootpointRegions': [],
                'SouthBTracedFootpointRegions': []
            })

            for x_coord in coords_element.findall(\
                    '{http://sscweb.gsfc.nasa.gov/schema}X'):

                result['Data'][data_i]['Coordinates']['X'].append(\
                    float(x_coord.text))

            for y_coord in coords_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}Y'):

                result['Data'][data_i]['Coordinates']['Y'].append(\
                    float(y_coord.text))

            for z_coord in coords_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}Z'):

                result['Data'][data_i]['Coordinates']['Z'].append(\
                    float(z_coord.text))

            for lat_coord in coords_element.findall(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Latitude'):

                result['Data'][data_i]['Coordinates']['Latitude'].append(\
                    float(lat_coord.text))

            for lon_coord in coords_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}Longitude'):

                result['Data'][data_i]['Coordinates']['Longitude'].append(\
                    float(lon_coord.text))

            for lt_coord in coords_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}LocalTime'):

                result['Data'][data_i]['Coordinates']['LocalTime'].append(\
                    float(lt_coord.text))

            for time in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}Time'):

                result['Data'][data_i]['Time'].append(\
                    dateutil.parser.parse(time.text))

            for b_trace_data in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}BTraceData'):

                result['Data'][data_i]['BTraceData'].append({
                    'CoordinateSystem': CoordinateSystem(b_trace_data.find(\
                         '{http://sscweb.gsfc.nasa.gov/schema}CoordinateSystem').text),
                    'Hemisphere': Hemisphere(b_trace_data.find(\
                         '{http://sscweb.gsfc.nasa.gov/schema}Hemisphere').text),
                    'Latitude': [],
                    'Longitude': [],
                    'ArcLength': []
                })
                for lat in b_trace_data.findall(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Latitude'):

                    result['Data'][data_i]['BTraceData'][-1]['Latitude'].append(\
                        float(lat.text))

                for lon in b_trace_data.findall(\
                    '{http://sscweb.gsfc.nasa.gov/schema}Longitude'):

                    result['Data'][data_i]['BTraceData'][-1]['Longitude'].append(\
                        float(lon.text))

                for arc_length in b_trace_data.findall(\
                    '{http://sscweb.gsfc.nasa.gov/schema}ArcLength'):

                    result['Data'][data_i]['BTraceData'][-1]['ArcLength'].append(\
                         float(arc_length.text))

            for radial_length in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}RadialLength'):

                result['Data'][data_i]['RadialLength'].append(\
                    float(radial_length.text))

            for magnetic_strength in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}MagneticStrength'):

                result['Data'][data_i]['MagneticStrength'].append(\
                    float(magnetic_strength.text))

            for neutral_sheet_distance in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}NeutralSheetDistance'):

                result['Data'][data_i]['NeutralSheetDistance'].append(\
                    float(neutral_sheet_distance.text))

            for bow_shock_distance in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}BowShockDistance'):

                result['Data'][data_i]['BowShockDistance'].append(\
                    float(bow_shock_distance.text))

            for magneto_pause_distance in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}MagnetoPauseDistance'):

                result['Data'][data_i]['MagnetoPauseDistance'].append(\
                    float(magneto_pause_distance.text))

            for dipole_l_value in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}DipoleLValue'):

                result['Data'][data_i]['DipoleLValue'].append(\
                    float(dipole_l_value.text))

            for dipole_invariant_latitude in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}DipoleInvariantLatitude'):

                result['Data'][data_i]['DipoleInvariantLatitude'].append(\
                    float(dipole_invariant_latitude.text))

            for spacecraft_region in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}SpacecraftRegion'):

                result['Data'][data_i]['SpacecraftRegion'].append(\
                    SpaceRegion(spacecraft_region.text))

            for radial_footpoint_region in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}RadialTracedFootpointRegions'):

                result['Data'][data_i]['RadialTracedFootpointRegions'].append(\
                    FootpointRegion(radial_footpoint_region.text))

            for b_gse_x in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}BGseX'):

                result['Data'][data_i]['BGseX'].append(\
                    float(b_gse_x.text))

            for b_gse_y in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}BGseY'):

                result['Data'][data_i]['BGseY'].append(\
                    float(b_gse_y.text))

            for b_gse_z in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}BGseZ'):

                result['Data'][data_i]['BGseZ'].append(\
                    float(b_gse_z.text))

            for b_traced_footpoint_region in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}NorthBTracedFootpointRegions'):

                result['Data'][data_i]['NorthBTracedFootpointRegions'].append(\
                    FootpointRegion(b_traced_footpoint_region.text))

            for b_traced_footpoint_region in data_element.findall(\
                   '{http://sscweb.gsfc.nasa.gov/schema}SouthBTracedFootpointRegions'):

                result['Data'][data_i]['SouthBTracedFootpointRegions'].append(\
                    FootpointRegion(b_traced_footpoint_region.text))

        #print(result)

        return result
    # pylint: enable=too-many-locals
    # pylint: enable=too-many-branches



#!/usr/bin/env python3

#
# NOSA HEADER START
#
# The contents of this file are subject to the terms of the NASA Open
# Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may
# not use this file except in compliance with the Agreement.
#
# You can obtain a copy of the agreement at
#   docs/NASA_Open_Source_Agreement_1.3.txt
# or
#   https://sscweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
#
# See the Agreement for the specific language governing permissions
# and limitations under the Agreement.
#
# When distributing Covered Code, include this NOSA HEADER in each
# file and include the Agreement file at
# docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the
# following below this NOSA HEADER, with the fields enclosed by
# brackets "[]" replaced with your own identifying information:
# Portions Copyright [yyyy] [name of copyright owner]
#
# NOSA HEADER END
#
# Copyright (c) 2013-2019 United States Government as represented by
# the National Aeronautics and Space Administration. No copyright is
# claimed in the United States under Title 17, U.S.Code. All Other
# Rights Reserved.
#

"""
Package for accessing the Satellite Situation Center (SSC) web services
https://sscweb.gsfc.nasa.gov/WebServices/REST/.
"""

import os
import platform
import xml.etree.ElementTree as ET
from datetime import datetime, timezone
from enum import Enum
#import logging
import httplib2
import dateutil.parser

#from sscws.timeinterval import TimeInterval
#from sscws.coordinates import *
#from sscws.bfieldmodels import *
#from sscws.outputoptions import *


__version__ = "0.1.1"
